/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/

#include "Pilot.h"
#include "librogue.h"

/* Unlike the original, this rogue port cannot show the whole screen
   at a time.  I think I need an x,y for the top,left visible
   character and a width,height in characters of the visible portion. 
   These should be updated as the rogue tries to walk off screen.  
   Required:
         visible_foo >= 0
	 visible_x + visible_w <= DCOLS
	 visible_y + visible_h <= DROWS
	 let's say visible_h, visible_w are constants
	 if w = DCOLS and h = DROWS, x = y = 0 as in normal rogue.
*/
/* The screen can have 15 rows, but I took 4 for messages and stats */
#define visible_h 11
/* You can put about 19.9 m's on a memo line.. so.. we'll have 20 cols */
#define visible_w 20
/* Height in pixels of one character ... in this case an 'M' */
#define visible_char_h 10
/* Width in pixels of one character ... 160 pixels / 20 col = 8 pix/col */
#define visible_char_w 8

/* This is the offset to convert character-positions in
   the "visible" part of the screen represented by 'terminal'
   to the larger "real" dungeon represented by 'buffer'. */
Short visible_x = 0, visible_y = 0;

/*
typedef struct wheres_cursor_s {
  Short _cury, _curx; // absolute, I guess
  Short _maxy, _maxx;
} wheres_cursor;
wheres_cursor scr_buf;
wheres_cursor *curscr = &scr_buf;
*/
Char terminal[visible_h][visible_w]; /* relative - a model of the screen */
Char buffer[DROWS][DCOLS]; /* absolute - a model of the dungeon */
Boolean screen_dirty;
Boolean lines_dirty[DROWS]; /* absolute */


/* If the rogue is off the visible screen, redraw centered on rogue.
   If the rogue is on the edge of the visible screen but NOT the virtual
   screen, scroll the visible screen (observing boundary conditions.) */

/* 
 *  This will clear the part of the display that belongs to
 *  "the dungeon".  It also clears 'terminal' since that is
 *  a representation of the display.
 *  This should be called before the offset for 'terminal' is
 *  moved to show a different part of 'buffer'.
 */
static void clear_visible() 
{
  Short i, j;

  /* Clear the physical screen. */
  RectangleType r;
  RctSetRectangle(&r, 0, 23, 160, 114);
  WinEraseRectangle(&r, 0);

  /* Update 'terminal' to represent the cleared screen. */
  screen_dirty = true;
  for (i = 0 ; i < visible_h; i++) {
    lines_dirty[i+visible_y] = true;
    for (j = 0; j < visible_w; j++) {
      /*       terminal[i][j] = buffer[i+visible_y][j+visible_x]; */
      terminal[i][j] = ' ';
    }
  }
}

/*
 *  This will ensure that the rogue remains "on screen",
 *  i.e. it scrolls 'terminal' so that it includes the rogue.
 */
void check_rogue_position(fighter * rogue) {
  if (rogue->col < visible_x ||
      rogue->col >= visible_x + visible_w ||
      rogue->row < visible_y ||
      rogue->row >= visible_y + visible_h) {
    /* I guess the rogue teleported or something */
    //    clear_visible(); // moved to move_visible_window
    move_visible_window(rogue->col, rogue->row, true);
    return;
  }
  /* The rogue is able to move off the screen from here, so scroll it. */
  if (0 != visible_x && rogue->col <= visible_x + 1) {
    clear_visible();
    visible_x = max(visible_x - visible_w / 2, 0);
  } else if (DCOLS - visible_w != visible_x &&
	     rogue->col >= visible_x + visible_w - 2) {
    clear_visible();
    visible_x = min(visible_x + visible_w / 2, DCOLS - visible_w);
  }
  if (0 != visible_y && rogue->row <= visible_y + 1) {
    clear_visible();
    visible_y = max(visible_y - visible_h / 2, 0);
  } else if (DROWS - visible_h != visible_y && 
	     rogue->row >= visible_y + visible_h - 2) {
    clear_visible();
    visible_y = min(visible_y + visible_h / 2, DROWS - visible_h);
  }
}

/*
 *  This will move 'terminal' either so that it's centered on the
 *  buffer-coordinate x,y, or so that the buffer-coordinate x,y is the
 *  left top position of 'terminal'.
 */
void move_visible_window(Short left_x, Short top_y, Boolean centered) {
  clear_visible();
  if (centered) {
    left_x -= visible_w / 2;
    top_y -= visible_h / 2;
  }
  if (left_x < 0) left_x = 0;
  if (top_y < 0) top_y = 0;
  if (left_x + visible_w > DCOLS) left_x = DCOLS - visible_w;
  if (top_y + visible_h > DROWS) top_y = DROWS - visible_h;
  visible_x = left_x;
  visible_y = top_y;
}
/* Is there room for this? */
void scroll_window(Short direction, Short steps) {
  Short left_x, top_y;
  clear_visible();
  left_x = visible_x;
  top_y = visible_y;
  if (steps <= 0) return; // avoid extreme silliness
  steps = min(steps, max(DCOLS, DROWS)); // avoid extreme silliness
  if (direction == NORTH || direction == NORTHWEST || direction == NORTHEAST)
    top_y = max(top_y - steps, 0); // bounds checking: must be at least 0
  if (direction == WEST  || direction == NORTHWEST || direction == SOUTHWEST)
    left_x = max(left_x - steps, 0);
  if (direction == SOUTH || direction == SOUTHWEST || direction == SOUTHEAST)
    top_y = min(top_y + steps, DROWS - visible_h); // bounds checking..
  if (direction == EAST  || direction == NORTHEAST || direction == SOUTHEAST)
    left_x = min(left_x + steps, DCOLS - visible_w);
  visible_x = left_x;
  visible_y = top_y;
}

/* first see if this works, then eliminate the j-loop like chuvmey.c #3 */
void save_vbuffer(Ptr p, Int *offsetp, Err *errorp)
{
  Int i;
  *offsetp = 0;
  for (i = 0 ; i < DROWS ; i++) {
    *errorp = DmWrite(p, *offsetp, buffer[i], DCOLS * sizeof(Char));
      *offsetp += DCOLS * sizeof(Char);
  }
}

void mvaddch(Short row, Short col, Short ch)
{
  buffer[row][col] = ch;
  lines_dirty[row] = true;
  screen_dirty = true;
}

/* row, col are RELATIVE here */
static void put_char_at(Short row, Short col, Char ch)
{
  Short cheat;
  /* the space has already been cleared with clear_line */
  /* on second thought, no it hasn't. */
  RectangleType r;
  RctSetRectangle(&r, col * visible_char_w, (row+2)*visible_char_h+5,
		  visible_char_w, visible_char_h); /* Left, Top, W, H */
  WinEraseRectangle(&r, 0); /* 0 for square corners */
  /* calculate pixel position of "row, col" and put char there */
  cheat = visible_char_w - FntCharWidth(ch);
  if (cheat <= 1)
    cheat = 0;
  else
    cheat /= 2;
  if (ch != ' ') {
    /* unfortunately, lower-case letters are a pain. */
    if (ch == 'g' || ch == 'j' || ch == 'p' || ch == 'q' || ch == 'y')
      WinDrawChars(&ch, 1, col * visible_char_w + cheat, 
		   (row+2) * visible_char_h+5-1);
    else 
      WinDrawChars(&ch, 1, col * visible_char_w + cheat, 
		   (row+2) * visible_char_h+5);
  }

  //    WinDrawInvertedChars(&ch, 1, col * visible_char_w + cheat, 
  //			         (row+2) * visible_char_h+5);

  terminal[row][col] = ch;

}
void refresh()
{
  Short col, line, absline;

  if (screen_dirty) {

    for (line = visible_h - 1; line >= 0; line--) {
      /* line is RELATIVE, line+visible_y is ABSOLUTE. */
      absline = line + visible_y;
      if (lines_dirty[absline]) {
	/* clear_row(line); */ /* XXXXXX */
	for (col = 0; col < visible_w; col++) {
	  /* col is RELATIVE, col+visible_x is ABSOLUTE. */
	  if (buffer[absline][col+visible_x] != terminal[line][col]) {
	    put_char_at(line, col, buffer[absline][col+visible_x]);
	  }
	}
	lines_dirty[absline] = false;
      }
    }
    screen_dirty = false;
  }
}

/* row, col are ABSOLUTE */
Short mvinch(Short row, Short col)
{
  return((Short) buffer[row][col]);
}

static void clear_buffers()
{
  Short i, j;

  screen_dirty = false;

  for (i = 0; i < DROWS; i++) {
    lines_dirty[i] = false;
    for (j = 0; j < DCOLS; j++) {
      buffer[i][j] = ' ';
    }
  }
  for (i = 0 ; i < visible_h; i++) {
    for (j = 0; j < visible_w; j++) {
      terminal[i][j] = ' ';
    }
  }
}

void clear() 
{
  RectangleType r;
  /* I draw lines at y=22 and y=137 */
  RctSetRectangle(&r, 0, 23, 160, 114); /* Left, Top, W, H */
  WinEraseRectangle(&r, 0); /* 0 for square corners */
  /*   cur_row = cur_col = 0; */
  //  move(0, 0);
  clear_buffers(); /* sets dirty = false */
  visible_x = visible_y = 0;
}
void very_clear() 
{
  WinEraseWindow();
  /*   cur_row = cur_col = 0; */
  //  move(0, 0);
  clear_buffers(); /* sets dirty = false */
  visible_x = visible_y = 0;
}

/* was used for dragon flame, not any more
void standout()
{
  buf_stand_out = true;
}
void standend()
{
  buf_stand_out = false;
} */

// this is now essentially identical to mvinch
Char peek_me(Short row, Short col) {
  if (0 <= row && 0 <= col && row < DROWS && col < DCOLS)
    return buffer[row][col];
  else
    return 0;
}

// Given a location that the player tapped at, return its (row,col)
// (The y is guaranteed to be within 23-136.)
Short what_row_col(Short x, Short y, Short *rowp, Short *colp) {
  Short col, row;
  col = x / visible_char_w;
  row = ((y - 5) / visible_char_h) - 2;
  col += visible_x;
  row += visible_y;
  if (col < 0) col = 0;
  if (row < 0) row = 0;
  if (col >= DCOLS) col = DCOLS-1;
  if (row >= DROWS) row = DROWS-1;
  *rowp = row;
  *colp = col;
  return buffer[row][col];
}
