/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/

#include "Pilot.h"
#include "librogue.h"

/***************************************************************
                   GET_DAMAGE
 IN:
 ds = representation of how many dice of what kind to roll.
 r = if true, don't roll; return the maximum possible damage.
 OUT:
 calculated damage
 PURPOSE:
 Calculate damage, either from rolling the dice represented
 by ds, or (if 'r') use the maximum possible.
****************************************************************/
Short get_damage(Char ds[4], Boolean r) {
  Short i = 0, j, n, d, total = 0;
  Boolean repeat = false;

  if (ds[1]==0) return 0; /* "no dice */
  if (!r) {
    /* don't roll, calc. max. damage */
     total = ds[0] * ds[1] + ds[2] * ds[3]; /* x d y + w d z */
     return total;
  }

  /* "do this one more time if there's a second pair" */
  do {
    repeat = !repeat; /* true first time, false second time */
    n = ds[i++];
    d = ds[i++];
    for (j = 0; j < n; j++) {
      total += get_rand(1, d);
    }
  } while (repeat && ds[3] != 0);

  return total;
}


/***************************************************************
                   DAMAGE_FOR_STRENGTH
 IN:
 sotu = various globals (rogue, add_strength)
 OUT:
 the damage modifier due to exceptional strength
 PURPOSE:
 Use this when the rogue hits something: if the rogue is
 exceptionally strong, extra damage is done; if the rogue is
 exceptionally weak, less than the normal amount will be inflicted.
****************************************************************/
// ELIMINATE
static Short damage_for_strength(struct state_of_the_union * sotu) {
  Short strength;

  strength = sotu->roguep->str_current + sotu->add_strength;

  if (strength <= 6) {
    return(strength-5);
  }
  if (strength <= 14) {
    return(1);
  }
  if (strength <= 17) {
    return(3);
  }
  if (strength <= 18) {
    return(4);
  }
  if (strength <= 20) {
    return(5);
  }
  if (strength <= 21) {
    return(6);
  }
  if (strength <= 30) {
    return(7);
  }
  return(8);
}
/*
 (str-10) / 3 = 
 <=  4 --> -2
 <=  7 --> -1
 <= 10 -->  0
 <= 13 -->  1
 <= 16 -->  2
 <= 19 -->  3
 <= 22 -->  4
 <= 25 -->  5
 <= 28 -->  6
*/


/***************************************************************
                   GET_HIT_CHANCE
 IN:
 sotu = various globals (rogue, ring_dex, r_rings)
 OUT:
 rogue's chance of hitting something
 PURPOSE:
 Used when the rogue tries to hit a monster, to determine
 the chance that the rogue actually will hit it.
****************************************************************/
// change for throwing?
Short get_hit_chance(object *weapon, Boolean thrown,
		     struct state_of_the_union * sotu) {
  Short hit_chance;
  Short to_hit = 1;

  // I'm not keen on the weapon->damage[0] here
  if (weapon) {
    to_hit = weapon->damage[0];
    if (!thrown && !(weapon->o_flags & O_MISSILE)) {
	to_hit += 3 * weapon->hit_enchant;
    }
    if ( thrown &&  (weapon->o_flags & O_MISSILE)) { 
      if ((weapon->launcher != NO_LAUNCHER)
	  && (sotu->roguep->weapon->which_kind == weapon->launcher)) {
	to_hit += 2 * (sotu->roguep->weapon->hit_enchant
		       + weapon->hit_enchant);
      }
      if ((weapon->launcher == NO_LAUNCHER)
	  && (weapon->in_use_flags & BEING_WIELDED)) {
	to_hit += 3 * weapon->hit_enchant;	
      }
    }
  }

  hit_chance = 50;
  hit_chance += to_hit;

  // placeholder for a dexterity bonus
  hit_chance += min( 5, ((sotu->roguep->str_current
			  + sotu->add_strength - 10))); // can be <0

  hit_chance += (((2 * sotu->roguep->exp) +
		  (2 * sotu->ring_dex)) - sotu->r_rings);
  return hit_chance;
  // if thrown and properly-launched, hit chance *= 4/3!
}

/***************************************************************
                   GET_WEAPON_DAMAGE
 IN:
 obj = the weapon being used (or 0)
 sotu = various globals (rogue, ring_dex, r_rings)
 OUT:
 The damage inflicted, including modifiers
 PURPOSE:
 Called after the rogue hits something, to calculate the total
 damage inflicted (including weapon, strength, & magical effects)
****************************************************************/
// REWRITE
// see roll_em in urogue.
Short get_weapon_damage(object *obj, Boolean thrown,
			struct state_of_the_union * sotu) {
  Short damage;
  object *use_my_bonus = obj;

  Char dice[4] = {0,0,0,0};
  if ((!obj) || (obj->what_is != WEAPON)) {
    // no weapon!  caller is clueless.  do 'hands' do any damage?
    damage = damage_for_strength(sotu);
    damage += ((((sotu->roguep->exp + sotu->ring_dex) - sotu->r_rings) + 1)/2);
    return (damage > 0 ? damage - 1 : 0);
  }

  // default is not-thrown, or improperly-launched-missile.
  dice[0] = obj->damage[0];
  dice[1] = obj->damage[1] + ( (!(obj->o_flags
				  & O_MISSILE)) ? obj->d_enchant : 0 );
  if (thrown) {
    if (!(obj->o_flags & O_MISSILE)) {
      // thrown non-missile?
      dice[0] = obj->damage[2];
      dice[1] = obj->damage[3]; // no hit+, no dam+
    } else {
      if (sotu->roguep->weapon) {
	if (obj->launcher != NO_LAUNCHER) {
	  // e.g. properly-launched arrow
	  if (sotu->roguep->weapon->which_kind == obj->launcher) {
	    use_my_bonus = sotu->roguep->weapon;
	    dice[0] = obj->damage[2];
	    dice[1] = obj->damage[3] + (sotu->roguep->weapon->d_enchant
					+ obj->d_enchant);
	  } 
	} else if (obj->in_use_flags & BEING_WIELDED) {
	  // e.g. properly-launched shuriken
	  dice[0] = obj->damage[2];
	  dice[1] = obj->damage[3] + obj->d_enchant;
	}
      }
    }
  }
  damage = get_damage(dice, 1);

  damage += damage_for_strength(sotu); // inline!
  damage += ((((sotu->roguep->exp + sotu->ring_dex) - sotu->r_rings) + 1) / 2);
  return max(damage,0);
}


/***************************************************************
                   GET_DIR_RC
 IN:
 dir = direction
 row,col = location (modified)
 allow_off_screen = whether to do bounds-checking
 OUT:
 nothing (really row,col)
 PURPOSE:
 Given a location and a direction, find the location that is
 one move away in the given direction (and return in row,col.)
 (If allow_off_screen is false, the location will be unchanged
 if the move would be outside the dungeon proper.)
****************************************************************/
void get_dir_rc(Short dir, Short *row, Short *col, 
		Boolean allow_off_screen) {
  switch(dir) {
  case WEST:
    if (allow_off_screen || (*col > 0)) {
      (*col)--;  /* 'h' */
    }
    break;
  case SOUTH:
    if (allow_off_screen || (*row < (DROWS-2))) {
      (*row)++;  /* 'j' */
    }
    break;
  case NORTH:
    if (allow_off_screen || (*row > MIN_ROW)) {
      (*row)--;  /* 'k' */
    }
    break;
  case EAST:
    if (allow_off_screen || (*col < (DCOLS-1))) {
      (*col)++;  /* 'l' */
    }
    break;
  case NORTHWEST:
    if (allow_off_screen || ((*row > MIN_ROW) && (*col > 0))) {
      (*row)--;  /* 'y' */
      (*col)--;
    }
    break;
  case NORTHEAST:
    if (allow_off_screen || ((*row > MIN_ROW) && (*col < (DCOLS-1)))) {
      (*row)--;  /* 'u' */
      (*col)++;
    }
    break;
  case SOUTHEAST:
    if (allow_off_screen || ((*row < (DROWS-2)) && (*col < (DCOLS-1)))) {
      (*row)++;  /* 'n' */
      (*col)++;
    }
    break;
  case SOUTHWEST:
    if (allow_off_screen || ((*row < (DROWS-2)) && (*col > 0))) {
      (*row)++;  /* 'b' */
      (*col)--;
    }
    break;
  }
  /* anything else, don't move. */
}
