/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/

#include "Pilot.h"
#include "librogue.h"

/***************************************************************
                   TRAP_AT
 IN:
 row,col = the location to check for traps
 sotu = various globals (traps)
 OUT:
 the type of trap, or NO_TRAP.
 PURPOSE:
 Determine whether there's a trap at the indicated location,
 and what type of trap it is.
****************************************************************/
Short trap_at(Short row, Short col, struct state_of_the_union *sotu) {
  short i;
  trap * traps = sotu->traps;

  /* Actually, it's the other way around - look through the
     array of traps and see if one of them is at this location */
  for (i = 0; ((i < MAX_TRAPS) && (traps[i].trap_type != NO_TRAP)); i++) {
    if ((traps[i].trap_row == row) && (traps[i].trap_col == col)) {
      return(traps[i].trap_type);
    }
  }
  return(NO_TRAP);
}

/***************************************************************
                   ADD_TRAPS
 IN:
 sotu = various globals (traps, rooms, dungeon, cur_level, party_room)
 OUT:
 nothing
 PURPOSE:
 Called when a level is created, to randomly hide some traps in it.
****************************************************************/
void add_traps(struct state_of_the_union *sotu) {
  Short i, n, tries = 0;
  Short row, col;
  trap * traps = sotu->traps;
  room * rooms = sotu->rooms;
  UShort ** dungeon = sotu->dungeon;

  /* Figure out how many traps to put on this level. */
  if (sotu->cur_level <= 2) {
    n = 0;
  } else if (sotu->cur_level <= 7) {
    n = get_rand(0, 2);
  } else if (sotu->cur_level <= 11) {
    n = get_rand(1, 2);
  } else if (sotu->cur_level <= 16) {
    n = get_rand(2, 3);
  } else if (sotu->cur_level <= 21) {
    n = get_rand(2, 4);
  } else if (sotu->cur_level <= (AMULET_LEVEL + 2)) {
    n = get_rand(3, 5);
  } else {
    n = get_rand(5, MAX_TRAPS);
  }
  /* For each trap, pick a type and try to put it somewhere. */
  for (i = 0; i < n; i++) {
    traps[i].trap_type = get_rand(0, (TRAPS - 1));

    /* A party room *always* gets a trap, if we can find a space for it. */
    if ((i == 0) && (sotu->party_room != NO_ROOM)) {
      do {
	row = get_rand((rooms[sotu->party_room].top_row+1),
		       (rooms[sotu->party_room].bottom_row-1));
	col = get_rand((rooms[sotu->party_room].left_col+1),
		       (rooms[sotu->party_room].right_col-1));
	tries++;
      } while (((dungeon[row][col] & (OBJECT|STAIRS|TRAP|TUNNEL)) ||
		(dungeon[row][col] == NOTHING)) && (tries < 15));
      if (tries >= 15) {
	gr_row_col(&row, &col, (FLOOR | MONSTER), sotu);
      }
    } else {
      gr_row_col(&row, &col, (FLOOR | MONSTER), sotu);
    }
    traps[i].trap_row = row;
    traps[i].trap_col = col;
    dungeon[row][col] |= (TRAP | HIDDEN);
  }
}

/***************************************************************
                   SHOW_TRAPS
 IN:
 sotu = various globals (dungeon)
 OUT:
 nothing
 PURPOSE:
 Make visible all the traps on this level (rogue used a magic item.)
****************************************************************/
void show_traps(struct state_of_the_union *sotu) {
  Short i, j;

  for (i = 0; i < DROWS; i++) {
    for (j = 0; j < DCOLS; j++) {
      if (sotu->dungeon[i][j] & TRAP) {
	mvaddch(i, j, '^');
      }
    }
  }
}
