/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/

#include "Pilot.h"
#include "librogue.h"

/***************************************************************
                   GET_ZAPPED_MONSTER
 IN:
 dir = direction of the long-distance attack
 row, col = location where the attack was initiated
 sotu = various globals (dungeon, level_monsters)
 OUT:
 0 if no monster affected (row,col = the wall or whatever)
 else, the monster affected (row,col = the location of the monster)
 PURPOSE:
 The character has performed a directional action (see get_thrown_at_monster)
 that is effective until it hits a wall or monster.  Follow the
 line of fire and find the monster, if any, that will be hit.
 (Note: a monster that's imitating something won't be hit...)
****************************************************************/
object * get_zapped_monster(Short dir, Short *row, Short *col,
			    Short ch, Boolean long_zap,
			    struct state_of_the_union *sotu) 
{
  Short orow, ocol, rrow, rcol;
  Short i;
  object *mon;

  rrow = *row; rcol = *col;
  for (i = 0 ; ; i++) {
    //    if ((ch && !long_zap) && (i > 0) && rogue_can_see(orow, ocol, sotu)) {
    //      mvaddch(orow, ocol, get_dungeon_char(orow, ocol, sotu));
    //      //SysTaskDelay(SysTicksPerSecond()/15); // XXX animation
    //      //refresh();
    //    }
    orow = *row; ocol = *col;
    /* step one square in the indicated direction */
    get_dir_rc(dir, row, col, false);
    /* if you've hit a wall or a monster, you're done */
    if (((*row == orow) && (*col == ocol)) ||
	(sotu->dungeon[*row][*col] & (HORWALL | VERTWALL)) ||
	(sotu->dungeon[*row][*col] == NOTHING)) {
      mon = 0;
      break;
    }
    if (ch && rogue_can_see(*row, *col, sotu)) {
      /* Draw the zap */
      if (!(sotu->dungeon[*row][*col] & MONSTER))
	mvaddch(*row, *col, ch);
      refresh(); // BEFORE the wait
      SysTaskDelay(SysTicksPerSecond()/15); // XXX animation
      /* Undraw the zap (if it's a short zap) */
      if (!long_zap && !(sotu->dungeon[*row][*col] & MONSTER))
	mvaddch(*row, *col, get_dungeon_char(*row, *col, sotu));
    }
    if (sotu->dungeon[*row][*col] & MONSTER) {
      if (!imitating(*row, *col, sotu)) {
	mon = object_at(sotu->level_monsters, *row, *col);
	break;
      }
    }
    /* haven't hit anything yet... */
  }
  /* "Zap"s can travel "forever", unlike thrown objects. */
  /* Now undraw the zap (if we drew it) */
  if (ch && long_zap) {
    for ( ; i > 0 ; i--) {
      get_dir_rc(dir, &rrow, &rcol, false);
      if (rogue_can_see(rrow, rcol, sotu)) {
	mvaddch(rrow, rcol, get_dungeon_char(rrow, rcol, sotu));
	refresh();
	SysTaskDelay(SysTicksPerSecond()/15); // XXX animation
      }
    }
  }
  return mon;
}

/***************************************************************
                   TELE_AWAY
 IN:
 monster = the affected monster
 sotu = various globals (dungeon, being_held, detect_monster)
 OUT:
 nothing
 PURPOSE:
 This will teleport the affected monster to a random location.
****************************************************************/
void tele_away(object *monster, struct state_of_the_union *sotu) {
  Short row, col;

  if (monster->m_flags1 & HOLDS) {
    sotu->being_held = false;
  }
  gr_row_col(&row, &col, (FLOOR | TUNNEL | STAIRS | OBJECT), sotu);
  mvaddch(monster->row, monster->col, monster->trail_char);
  sotu->dungeon[monster->row][monster->col] &= ~MONSTER;
  monster->row = row; 
  monster->col = col;
  sotu->dungeon[row][col] |= MONSTER;
  monster->trail_char = mvinch(row, col);
  if (sotu->detect_monster || rogue_can_see(row, col, sotu)) {
    mvaddch(row, col, gmc(monster, sotu));
  }
}
