/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/

#ifndef __LIBROGUE_H__
#define __LIBROGUE_H__

#include "../rogue_defines.h"

Boolean bite_me(Short x);

/* random.c */
void seed_random();
Int get_rand(Int x, Int y);
Boolean rand_percent(Int percentage);
Boolean coin_toss();
void rand_around(Short i, Short *r, Short *c);

/* hit.c */
Short get_damage(Char ds[4], Boolean r);
//Short get_w_damage(object *obj); /* used in hit.c only */
//Short to_hit(object *obj); /* ditto */
//Short damage_for_strength(struct state_of_the_union * sotu); /* ditto */
Short get_hit_chance(object *weapon, Boolean thrown,
		     struct state_of_the_union * sotu);
Short get_weapon_damage(object *weapon, Boolean thrown,
			struct state_of_the_union * sotu);
void get_dir_rc(Short dir, Short *row, Short *col, 
		Boolean allow_off_screen);

/* level.c */
//void make_room(Short rn, Short r1, Short r2, Short r3,
//	       struct state_of_the_union * sotu);
void put_door(room * rm, Short dir, Short * row, Short * col,
	      struct state_of_the_union * sotu);
Boolean mask_room(Short rn, Short * row, Short * col, UShort mask,
	      struct state_of_the_union * sotu);
void hide_boxed_passage(Short row1, Short col1,
			Short row2, Short col2, 
			Short n,
			struct state_of_the_union * sotu);
void draw_simple_passage(Short row1, Short col1, 
			 Short row2, Short col2,
			 Short dir,
			 struct state_of_the_union * sotu);
Boolean drop_check(struct state_of_the_union * sotu);
Short hp_raise(struct state_of_the_union * sotu);
Boolean connect_rooms(Short room1, Short room2, 
		      struct state_of_the_union * sotu);
void clear_level(struct state_of_the_union * sotu);
void put_player(Short nr, struct state_of_the_union *sotu);
//Short get_exp_level(Long e, struct state_of_the_union *sotu);
void add_exp(Int e, Boolean promotion, struct state_of_the_union *sotu);

/* my_message.c */
void check_message();
void message(Char * msg, struct state_of_the_union * sotu);
void print_stats(UInt stat_mask, struct state_of_the_union * sotu);


/* my_curses.c */
void check_rogue_position(fighter * rogue);
void move_visible_window(Short left_x, Short top_y, Boolean centered);
void scroll_window(Short direction, Short steps);
void save_vbuffer(Ptr p, Int *offsetp, Err *errorp);
//void move(Short row, Short col);
//void addch(Char ch);
void mvaddch(Short row, Short col, Short ch);
void refresh();
Short mvinch(Short row, Short col);
void clear();
//void standout();
//void standend();
/* break abstraction barrier for map of dungeon */
Char peek_me(Short row, Short col);
Short what_row_col(Short screen_x, Short screen_y, Short *rowp, Short *colp);

/* monster.c */
void copy_mon_name(Short index, Char *foo);
void print_mon_name(Short index, struct state_of_the_union *sotu);
void mon_name(object *monster, Char *foo, struct state_of_the_union *sotu);
/* const Char * mon_name(object *monster, struct state_of_the_union *sotu); */
Boolean rogue_is_around(Short row, Short col, fighter *rogue);
void wake_up(object *monster);
Char gr_obj_char();
Boolean mon_sees(object *monster, Short row, Short col, room * rooms);
Char gmc(object *monster, struct state_of_the_union * sotu);
void aim_monster(object *monster, room * rooms); /* only used one place */
Boolean rogue_can_see(Short row, Short col, struct state_of_the_union * sotu);
Boolean no_room_for_monster(Short rn, room * rooms, UShort ** dungeon);
Boolean mon_can_go(object *monster, Short row, Short col,
		   struct state_of_the_union * sotu);
void wake_room(Short rn, Boolean entering, Short row, Short col,
	       struct state_of_the_union *sotu);
void show_monsters(struct state_of_the_union *sotu);
void aggravate(struct state_of_the_union *sotu);
Char gmc_row_col(Short row, Short col, struct state_of_the_union *sotu);
void move_mon_to(object *monster, Short row, Short col,
		 struct state_of_the_union *sotu);
Boolean mtry(object *monster, Short row, Short col,
	     struct state_of_the_union *sotu);
Boolean move_confused(object *monster, struct state_of_the_union *sotu);
Boolean flit(object *monster, struct state_of_the_union *sotu);

/* room.c */
Short get_room_number(Short row, Short col, room * rooms);
Char get_mask_char(UShort mask);
Short gr_room(room * rooms);
//Boolean get_oth_room(Short rn, Short *row, Short *col, room * rooms);
void dr_course(object *monster, Boolean entering, Short row, Short col,
	       struct state_of_the_union * sotu);
//Boolean is_all_connected(room * rooms, Boolean * rooms_visited);
//void visit_rooms(Short rn, room * rooms, Boolean * rooms_visited);
Char get_dungeon_char(Short row, Short col,
		      struct state_of_the_union * sotu);
void light_up_room(Short rn, struct state_of_the_union *sotu);
void light_passage(Short row, Short col, struct state_of_the_union *sotu);
void darken_room(Short rn, struct state_of_the_union *sotu);
void gr_row_col(Short *row, Short *col, UShort mask,
		struct state_of_the_union *sotu);
void draw_magic_map(struct state_of_the_union *sotu); /* YAY! */
Short party_objects(Short rn, struct state_of_the_union *sotu);

/* move.c */
Boolean can_move(Short row1, Short col1, Short row2, Short col2,
		 UShort ** dungeon);
Boolean is_passable(Short row, Short col, UShort ** dungeon);
Boolean next_to_something(Short drow, Short dcol,
			  struct state_of_the_union * sotu);

/* object.c */
object * object_at(object *pack, Short row, Short col);
Boolean is_vowel(Short ch);
struct id * get_id_table(object *obj, struct state_of_the_union * sotu);
void free_object(object *obj, struct state_of_the_union * sotu);
object * alloc_object(struct state_of_the_union * sotu);
void put_gold(struct state_of_the_union *sotu); 
//void plant_gold(Short row, Short col, Boolean is_maze,
//		struct state_of_the_union *sotu);
void place_at(object * obj, Short row, Short col,
	      struct state_of_the_union *sotu);
void free_stuff(object *objlist, struct state_of_the_union *sotu);
void free_old_level(struct state_of_the_union *sotu);
void put_stairs(struct state_of_the_union *sotu);

//UShort gr_what_is();
//void gr_scroll(object *obj);
//void gr_potion(object *obj);
//void gr_weapon(object *obj, Boolean assign_wk);
//void gr_armor(object *obj);
//void gr_wand(object *obj);
void get_food(object *obj, Boolean force_ration);
//void gr_ring(object *ring, Boolean assign_wk); /* was ring.c */
object * gr_object(struct state_of_the_union *sotu);
object * get_nth_object(Short n, object *pack); /* I made this */
void new_object_for_wizard(Short chtype, Short whatkind,
			   struct state_of_the_union *sotu);
/* Char * name_of(object *obj, struct state_of_the_union *sotu); */
void copy_name_of(object *obj, Char *foo,
		  struct state_of_the_union *sotu);
void get_desc(object *obj, Char *desc, struct state_of_the_union *sotu);
void get_desc_brief(object *obj, Char *desc, struct state_of_the_union *sotu);

void flop_weapon(object *weapon, Short row, Short col,
		 struct state_of_the_union *sotu); /* was in throw.c */
//object * get_thrown_at_monster(object *obj, Short dir, Short *row, Short *col,
//			       struct state_of_the_union *sotu); /* ditto */

/* pack.c */
//object * check_duplicate(object *obj, object *pack);
void take_from_pack(object *obj, object *pack);
Boolean mask_pack(object *pack, UShort mask);
/* Boolean has_amulet(); */ /* oops. */
Short get_armor_class(struct state_of_the_union * sotu);
//Char next_avail_ichar(struct state_of_the_union * sotu);
object * add_to_pack(object *obj, object *pack, Boolean condense,
		     struct state_of_the_union * sotu);
void unwear(object *obj, fighter *rogue);
void do_wear(object *obj, fighter *rogue);
void do_wield(object *obj, fighter *rogue);
void unwield(object *obj, fighter *rogue);
Short pack_count(object *new_obj, fighter *rogue, Short *weight);
Short max_weight(struct state_of_the_union * sotu);

/* spec_hit.c */
void get_closer(Short *row, Short *col, Short trow, Short tcol);
Boolean imitating(Short row, Short col, struct state_of_the_union * sotu);
void disappear(object *monster, struct state_of_the_union *sotu);
//Boolean try_to_cough(Short row, Short col, object *obj,
//		     struct state_of_the_union *sotu);
void cough_up(object *monster, struct state_of_the_union *sotu);
void check_gold_seeker(object *monster);
Boolean check_imitator(object *monster, struct state_of_the_union *sotu);

/* machdep.c */
Char * md_malloc(Int n);

/* traps.c */
Short trap_at(Short row, Short col, struct state_of_the_union *sotu);
void add_traps(struct state_of_the_union *sotu);
void show_traps(struct state_of_the_union *sotu);

/* use.c */
void hold_monster(struct state_of_the_union *sotu);
void tele(struct state_of_the_union *sotu);
void hallucinate(struct state_of_the_union *sotu);
void unhallucinate(struct state_of_the_union *sotu);
void unblind(struct state_of_the_union *sotu);
void relight(struct state_of_the_union *sotu);
void go_blind(struct state_of_the_union *sotu);
void confuse(struct state_of_the_union *sotu);
void unconfuse(struct state_of_the_union *sotu);
void uncurse_all(struct state_of_the_union *sotu);
Char * get_ench_color(struct state_of_the_union *sotu);
//void potion_heal(Boolean extra, struct state_of_the_union *sotu);
void vanish(object *obj, object *pack, struct state_of_the_union *sotu);
Boolean quaff(Short index, struct state_of_the_union *sotu);
/*
Boolean read_scroll(Short index, struct state_of_the_union *sotu,
		    Boolean * identify);
		    */
Boolean eat(Short index, struct state_of_the_union *sotu);

/* zap.c */
object * get_zapped_monster(Short dir, Short *row, Short *col,
			    Short ch, Boolean long_zap,
			    struct state_of_the_union *sotu);
void tele_away(object *monster, struct state_of_the_union *sotu);

/* ring.c */
void ring_stats(Boolean pr, struct state_of_the_union *sotu);
//void do_put_on(object *ring, Short index, struct state_of_the_union *sotu);
void un_put_on(object *ring, struct state_of_the_union *sotu);
Boolean remove_ring(Short index, struct state_of_the_union *sotu);
Boolean put_on_ring(Short index, Short finger_index, 
		    struct state_of_the_union *sotu);


#endif

