/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/

/*
 * chuvmey_nolib.c --- Leftovers that don't (directly) use the lib
 * If any.
 */

#include "Pilot.h"
#include "callback.h"
#include "iRogueRsc.h"
#include "Globals.h"

#include <SystemMgr.rh> // for user name
#include <DLServer.h> // for user name
#include <SoundMgr.h> // for feeping
#include <Preferences.h> // for feeping

#include "rogue.h"

// Jump-Island type Dependencies:
// get_username: md_malloc
// show_dungeon: peek_me
// save_character: save_vbuffer
// I'll think about "load_character*" another time
// (and, hell, all the rest of that.)

/* check out that database, dude! */
void save_vbuffer2(Ptr p, Int *offsetp, Err *errorp);
Char * md_malloc2(Int n);
//void init_fruit();
void mvaddch2(Short row, Short col, Short ch);
Char peek_me2(Short row, Short col);
Char specialized_labels[13][9] = {
  "NO-OP",
  "drop",
  "eat",
  "read",
  "put on",
  "wield",
  "zap",
  "quaff",
  "wear",
  "remove",
  "throw",
  "unwield",
  "unwear" //  "take off"
};
object * get_nth_object2(Short n, object *pack);





/* written late at night, needs sanity check */
void usr_form_init(FormPtr frm, struct state_of_the_union *sotu) {
  FieldPtr fld;
  CharPtr p;
  VoidHand vh;
  fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_usr));
  vh = (VoidHand) FldGetTextHandle(fld);
  if (!vh) {
    vh = MemHandleNew(32 * sizeof(Char));
  }
  FldSetTextHandle(fld, (Handle) 0);
  p = MemHandleLock(vh);
  StrNCopy(p, sotu->username, 30);
  MemHandleUnlock(vh);
  FldSetTextHandle(fld, (Handle) vh);
}
/* written late at night, needs sanity check */
void usr_form_update(FormPtr frm, struct state_of_the_union *sotu) {
  FieldPtr fld;
  CharPtr p;
  VoidHand vh;
  fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_usr));
  vh = (VoidHand) FldGetTextHandle(fld);
  if (vh) {
    p = MemHandleLock(vh);
    StrNCopy(sotu->username, p, 30);
    MemHandleUnlock(vh);
  }
}



/**********************************************************************
                       EXPLAIN_IT
 IN:
 x,y that the player tapped on
 various globals
 OUT:
 nothing
 PURPOSE:
 display what the thing is
**********************************************************************/
void message2(Char * msg, struct state_of_the_union * sotu);
Short what_row_col2(Short x, Short y, Short *rowp, Short *colp);
void print_mon_name2(Short index, struct state_of_the_union *sotu);
void explain_it(Short x, Short y, struct state_of_the_union *sotu) {
  Short row, col, mch, ch;
  //  Char buf[10];

  if (y <= 22)
    message2("message window", sotu);
  else if (y >= 137)
    message2("vital statistics of your rogue", sotu);
  else {
    ch = what_row_col2(x, y, &row, &col);
    //    ch = mvinch2(row, col);
    if (Is_Alpha(ch)) {
      if (ch > 'Z')
	mch = 26 + (ch - 'a'); // 97=='a' 65=='A'
      else
	mch = ch - 'A';
      print_mon_name2(mch, sotu);
    } else {
      switch(ch) {
      case ' ':
	message2("nothing is visible there", sotu);
	return;
      case '?':
	message2("scroll", sotu);
	return;
      case '!':
	message2("potion", sotu);
	return;
      case '*':
	message2("some gold", sotu);
	return;
      case -80:
	// FOODCHAR
	message2("some food", sotu);
	return;
      case '/':
	message2("wand", sotu);
	return;
      case ']':
	message2("armor", sotu);
	return;
      case ')':
	message2("weapon", sotu);
	return;
      case '=':
	message2("ring", sotu);
	return;
      case ',':
	message2("amulet", sotu);
	return;
      case '.':
	message2("the floor", sotu);
	return;
      case '+':
	message2("door", sotu);
	return;
      case '#':
	message2("tunnel", sotu);
	return;
      case '|':
      case -105:
	// EMDASH
	message2("wall", sotu);
	return;
      case '^':
	message2("trap", sotu);
	return;
      case '%':
	message2("stairs", sotu);
	return;
      case '@':
	message2("that's you", sotu);
	return;
      default:
	//	StrPrintF(buf, "%d", ch);
	message2("...beats me", sotu);
	//      message2(buf, sotu);
	return;
      }
    }
  }
}
/**********************************************************************
                       CALCULATE_WARNING
 IN:
 various globals
 OUT:
 warning level: 0-5.
 PURPOSE:
 Called when the character has a ring of warning, to determine the
 level of a$$-kicking-ness nearby.
 **********************************************************************/
#define WARN_AGAIN 75
static const Char warn_colors[6][7] = {
  "white", "pink", "red", "ruby", "purple", "black"
};
void calculate_warning(struct state_of_the_union *sotu) {
  object *monster;
  Short r, c, dist_sq;
  Short warn_lev = 0;
  Char glowbuf[30];

  monster = level_monsters->next_monster;
  while (monster) {
    // if the monster is within 10 squares of the rogue, it's of interest.
    // add something proportional to its 'level'.
    r = monster->row - sotu->roguep->row;
    c = monster->col - sotu->roguep->col;
    dist_sq = r * r + c * c; // square of the distance to the rogue
    if (dist_sq <= 100) {
      r = monster->first_level / 4;
      if (r > warn_lev)
	warn_lev += r;
    }
    monster = monster->next_monster;
  }
  // Maybe I should also divide as the rogue gains experience.
  if (warn_lev > 5)
    warn_lev = 5;

  // Done calculating the warning level.  Now see if we need to warn.

  if ((sotu->warning_moves++ > WARN_AGAIN)
      || (warn_lev > sotu->warning_level)) {
    sotu->warning_moves = 0;
    sotu->warning_level = (Char) warn_lev;
    StrPrintF(glowbuf, "your ring glows %s",
	      warn_colors[warn_lev]);
    message2(glowbuf, sotu);
    if (warn_lev > 0)
      sotu->interrupted = true;
  }

}

/**********************************************************************
                       SAVE_CHARACTER_ID
 IN:
 kind = one of the id_foo arrays
 kindnum = the number of items in that array
 index = the database index to pick up at
 OUT:
 nothing
 PURPOSE:
 Called when the character data is being saved to database records.
 This routine saves the various "id_foo" arrays.
 **********************************************************************/
static void save_character_id(struct id * kind, Short kindnum, UInt index)
{
  Err error;
  VoidHand fooRec;
  Ptr p;
  Int i, offset, size;
  Short len1, len2;

  size = 0;
  for (i = 0 ; i < kindnum ; i++) {
    size += sizeof(Short) * 2; /* the two string lengths (oh what the hell) */
    size += sizeof(Short) + sizeof(UShort); /* value, id_status */
    size += StrLen(kind[i].title) + 2; /* \0, and add one if odd */
    size += StrLen(kind[i].real) + 2; /* ditto (maintain alignment) */
  }

  fooRec = DmNewRecord(RogueDB, &index, size);
  p = MemHandleLock(fooRec);
  offset = 0;
  for (i = 0 ; i < kindnum ; i++) {
    len1 = StrLen(kind[i].title) + 1;
    if (len1 % 2 != 0) len1++; /* maintain alignment. */
    len2 = StrLen(kind[i].real) + 1;
    if (len2 % 2 != 0) len2++; /* any unused space is 0.. I think. */
    error = DmWrite(p, offset, &len1, sizeof(Short));
    offset += sizeof(Short);
    error = DmWrite(p, offset, &len2, sizeof(Short));
    offset += sizeof(Short);
    error = DmWrite(p, offset, &(kind[i].value), sizeof(Short));
    offset += sizeof(Short);
    error = DmWrite(p, offset, &(kind[i].id_status), sizeof(UShort));
    offset += sizeof(UShort);
    error = DmWrite(p, offset, kind[i].title, len1 * sizeof(Char));
    offset += len1 * sizeof(Char);
    error = DmWrite(p, offset, kind[i].real, len2 * sizeof(Char));
    offset += len2 * sizeof(Char);
  }
  MemPtrUnlock(p);

  DmReleaseRecord(RogueDB, index, true);
}
/**********************************************************************
                       SAVE_CHARACTER_OBJS
 IN:
 start = the start of a list of items to save
 indexp = the current database record index
 OUT:
 number of objects written to database (also, indexp is updated)
 PURPOSE:
 Save the items in the 'start' list (level_objects/level_monsters/pack)
 **********************************************************************/
/* for each object, write a record.  return number written.  */
/* BUG: items worn get that stuck on their name forever?! how? */
static UInt save_character_objs(object * start, UInt *indexp)
{
  Err error;
  VoidHand fooRec;
  Ptr p;
  object * tmp;
  object * obj = start->next_object;
  UInt x = 0;

  while (obj) {

    fooRec = DmNewRecord(RogueDB, indexp, sizeof(object));
    p = MemHandleLock(fooRec);
    tmp = obj->next_object;
    obj->next_object = (object*) 0;
    error = DmWrite(p, 0, obj, sizeof(object));
    obj->next_object = tmp;
    MemPtrUnlock(p);
    DmReleaseRecord(RogueDB, *indexp, true);
    
    (*indexp)++;
    obj = obj->next_object;
    x++;
    
  }

  return x;
}
/**********************************************************************
                       LOAD_CHARACTER_ID
 IN:
 idarray = one of the id_foo arrays
 num = the number of items in that array
 index = the database index to pick up at
 OUT:
 nothing
 PURPOSE:
 Called when the character data is being loaded from database records.
 This routine loads the various "id_foo" arrays.
 **********************************************************************/
static void load_character_id(struct id * idarray, Short num, UInt index) 
{
  VoidHand vh;
  VoidPtr p;
  Short i;
  Short * short_p;
  UShort * ushort_p;
  Char * char_p;
  
  Short len1, len2;

  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    short_p = (Short *) p;
    for (i = 0 ; i < num ; i++) {
      len1 = short_p[0];
      len2 = short_p[1];
      idarray[i].value = short_p[2];
      ushort_p = (UShort *) (short_p + 3);
      idarray[i].id_status = ushort_p[0];
      char_p = (Char *) (ushort_p + 1);
      MemMove(idarray[i].title, char_p, sizeof(Char) * len1);
      char_p += len1;
      MemMove(idarray[i].real, char_p, sizeof(Char) * len2);
      char_p += len2;
      short_p = (Short *) char_p;
    }
  }
  MemHandleUnlock(vh);

}


/**********************************************************************
                       LOAD_CHARACTER_OBJS
 IN:
 start = the start of a list of items to load
 indexp = the current database record index
 num = the number of items to load
 OUT:
 nothing (well, indexp is updated)
 PURPOSE:
 Load the items into the 'start' list (level_objects/level_monsters/pack)
 **********************************************************************/
/* for each object, write a record.  return number written.  */
static void load_character_objs(object * start, UInt *indexp, Short num)
{
  VoidHand vh;
  VoidPtr p;
  Short i;
  object *obj, *prev_obj;

  prev_obj = start;
  for (i = 0 ; i < num ; i++) {
    vh = DmQueryRecord(RogueDB, (*indexp)); /* read-only, promise */
    p = MemHandleLock(vh);
    if (p) {
      obj = (object *) md_malloc2(sizeof(object));
      prev_obj->next_object = obj;
      MemMove(obj, (object *) p, sizeof(object));
      obj->next_object = (object *) 0; /* need to re-zero pointers */
      prev_obj = obj;
    }
    MemHandleUnlock(vh);
    (*indexp)++;
  }
}


/**********************************************************************
                       FIX_VERBED_OBJS
 IN:
 pack = the list of objects the rogue is holding
 rogue
 OUT:
 nothing
 PURPOSE:
 Called when the rogue data is loaded from database records;
 this updates some pointers (from the rogue into the pack list.)
 Hooray for redundancy.
 **********************************************************************/
static void fix_verbed_objs(object *pack, fighter *rogue) {
  /* for each item in pack,
     if worn, wielded, or put on ... */
  object *obj;
  ULong ring_flag;
  Short i;
  obj = pack->next_object;
  while (obj) {
    switch(obj->what_is) {
    case RING:
      if (obj->in_use_flags & ON_EITHER_HAND) {
	ring_flag = obj->in_use_flags >> 2; // =~ divide by 4
	// Right-shift it into nothing; the index is #shifts - 1
	// (I think!  XXXX)
	for (i = 0 ; (i < 8) && ring_flag ; i++) {
	  ring_flag = ring_flag >> 1;
	  if (!ring_flag)
	    rogue->rings[i] = obj;
	}
      } // else not being worn.
      break;
    case WEAPON:
      if (obj->in_use_flags & BEING_WIELDED) {
	rogue->weapon = obj;
      }
      break;
    case ARMOR:
      if (obj->in_use_flags & BEING_WORN) {
	rogue->armor = obj;
      }
      break;
    }
    obj = obj->next_object;
  }
}

/**********************************************************************
                       SAVE_CHARACTER
 IN:
 sotu = various globals (all of 'em)
 OUT:
 nothing
 PURPOSE:
 Called when a game is in progress, i.e. the rogue is currently alive,
 and player 'preferences' are to save the game between sessions.
 This will basically write all the interesting data from the heap to a
 bunch of database records, which will be loaded into the heap again,
 the next time the player starts the Rogue application.
 It is very long, icky, and inefficient.
 **********************************************************************/
void save_character(struct state_of_the_union * sotu) {
  
  /* 5 Boolean, 8 Char, 14 Short */
  /*
  Boolean being_held   see_invisible   detect_monster   wizard   score_only
  Char hunger_str[8]
  Short cur_level max_level cur_room party_room r_de m_moves foods
        party_counter bear_trap halluc blind confused levitate haste_self
  */
  /* Plus all the stuff that's pointed to. */

  Err error;
  VoidHand fooRec;
  UInt index = 0;
  Ptr p;
  //  UInt attr; /* category */
  UInt object_count[4];
  //  Int i, j, offset;
  //  Char tmpc;
  Int i, offset;

  /* 
   *   0th:
   *   I ought to nuke anything that's in the database already....
   */

  /* 
   * FIRST:
   *   Save the 'sotu' record.
   *   Including pointers - we'll zero those on reload.
   */
  //  message("first", sotu);

  index = 10; /* was 0, but now 0-9 are Top10 records.. easier.. */

  fooRec = DmNewRecord(RogueDB, &index, sizeof(struct state_of_the_union));
  p = MemHandleLock(fooRec);
  error = DmWrite(p, 0, sotu, sizeof(struct state_of_the_union));
  /* I expected this not to work, but it does, I think. */
  MemPtrUnlock(p);

  /* Set the category of the record, for jollies.
  DmRecordInfo (RogueDB, index, &attr, NULL, NULL);
  attr |= dmUnfiledCategory;
  DmSetRecordInfo (RogueDB, index, &attr, NULL);   */

  /* Release the record, dirty=true. */
  DmReleaseRecord(RogueDB, index, true);

  /* 
   * SECOND:
   *   Save the 'rogue' record.
   *   Including pointers - we'll zero those on reload.
   */
  index += 1;

  fooRec = DmNewRecord(RogueDB, &index, sizeof(fighter));
  p = MemHandleLock(fooRec);
  error = DmWrite(p, 0, sotu->roguep, sizeof(fighter));
  MemPtrUnlock(p);

  DmReleaseRecord(RogueDB, index, true);

  /* 
   * THIRD:
   *   Save the 'dungeon' array.
   */
  index += 1;

  fooRec = DmNewRecord(RogueDB, &index, sizeof(UShort) * DROWS * DCOLS);
  p = MemHandleLock(fooRec);
  offset = 0;
  //  for (i = 0 ; i < DROWS ; i++) {
  //    for (j = 0 ; j < DCOLS ; j++) {
  //      /* there ought to be a better way */
  //      error = DmWrite(p, offset, &(sotu->dungeon[i][j]), sizeof(UShort));
  //      offset += sizeof(UShort);
  //    }
  for (i = 0 ; i < DROWS ; i++) {
    error = DmWrite(p, offset, sotu->dungeon[i], DCOLS * sizeof(UShort) );
    offset += DCOLS * sizeof(UShort);
  }
  MemPtrUnlock(p);

  DmReleaseRecord(RogueDB, index, true);

  /* 
   * THIRD AND A HALF:
   *   Save the 'buffer' array - this will be colossally slow
   */
  index += 1;

  fooRec = DmNewRecord(RogueDB, &index, sizeof(Char) * DROWS * DCOLS);
  p = MemHandleLock(fooRec);
  offset = 0;
  //    for (i = 0 ; i < DROWS ; i++) {
  //      for (j = 0 ; j < DCOLS ; j++) {
  //        tmpc = peek_me(i, j);
  //        /* maybe I could just have the library write it */
  //        error = DmWrite(p, offset, &(tmpc), sizeof(Char));
  //        offset += sizeof(Char);
  //      }
  //    }
  save_vbuffer2(p, &offset, &error);
  MemPtrUnlock(p);

  DmReleaseRecord(RogueDB, index, true);

  /* 
   * FOURTH:
   *   Save the 'rooms' array.
   */
  index += 1;

  fooRec = DmNewRecord(RogueDB, &index, sizeof(room) * MAXROOMS);
  p = MemHandleLock(fooRec);
  offset = 0;
  for (i = 0 ; i < MAXROOMS ; i++) {
    error = DmWrite(p, offset, &(sotu->rooms[i]), sizeof(room));
    offset += sizeof(room);
  }
  MemPtrUnlock(p);

  DmReleaseRecord(RogueDB, index, true);

  /* 
   * FIFTH:
   *   Save the 'traps' array (teeny tiny)
   */
  index += 1;

  fooRec = DmNewRecord(RogueDB, &index, sizeof(trap) * MAX_TRAPS);
  p = MemHandleLock(fooRec);
  offset = 0;
  for (i = 0 ; i < MAX_TRAPS ; i++) {
    error = DmWrite(p, offset, &(sotu->traps[i]), sizeof(trap));
    offset += sizeof(trap);
  }
  MemPtrUnlock(p);

  DmReleaseRecord(RogueDB, index, true);

  /* 
   * SIXTH:
   *   Save the 'id_FOO' arrays - they're all the same struct type
   *   scrolls, potions, wands, rings, weapons, armors
   *      better have a record for each... size limitation...
   */
  save_character_id(sotu->id_scrolls, SCROLLS, ++index);
  save_character_id(sotu->id_potions, POTIONS, ++index);
  save_character_id(sotu->id_wands, WANDS, ++index);
  save_character_id(sotu->id_rings, RINGS, ++index);
  //  save_character_id(sotu->id_weapons, WEAPONS, ++index);
  //  save_character_id(sotu->id_armors, ARMORS, ++index);
  /* seems to be working so far.... */

  /* 
   * SEVENTH:
   *   Save the 'rogue->pack', 'level_monsters', and 'level_objects'
   */
  index += 1;
  offset = index; /* yes, I'm skipping a record on purpose. */
  index += 1;
  object_count[0] = save_character_objs(&(sotu->roguep->pack), &index);
  object_count[1] = save_character_objs(sotu->level_monsters, &index);
  object_count[2] = save_character_objs(sotu->level_objects, &index);
  object_count[3] = 0;

  /* 
   * SIXTH and a half
   *   Save the 'is_wood' array... along with object-count info ...
   */
  index = offset; /* (restore skipped index from temporary) */

  fooRec = DmNewRecord(RogueDB, &index, 
		       sizeof(UInt) * 4 + sizeof(Boolean) * WANDS);
  p = MemHandleLock(fooRec);
  offset = 0;
  /* now write the number of pack items, level monsters, level items,
     and a 0 just to keep things tidy */
  for (i = 0 ; i < 4 ; i++) {
    error = DmWrite(p, offset, &(object_count[i]), sizeof(UInt));
    offset += sizeof(UInt);
  }
  /* ok, go ahead with the is_wood */
  for (i = 0 ; i < WANDS ; i++) {
    error = DmWrite(p, offset, &(sotu->is_wood[i]), sizeof(Boolean));
    offset += sizeof(Boolean);
  }
  MemPtrUnlock(p);

  /* THAT IS ALL.  I think. */

}

/* check out that database, dude! */
/**********************************************************************
                       LOAD_CHARACTER
 IN:
 sotu = various globals (all of 'em)
 OUT:
 nothing
 PURPOSE:
 As promised, this will load stuff from the database records (which
 will then be deleted) into the heap.  (It is called when such records
 exist and the application has just been started.)
 Long and tedious.
 **********************************************************************/
void load_character(struct state_of_the_union * sotu) {
  VoidHand vh;
  VoidPtr p;

  struct state_of_the_union *load_sotu;
  fighter * load_rogue;
  UShort * load_dungeon;
  room * load_room;
  trap * load_trap;
  Char * load_buf;
  UInt * uint_p;
  Boolean * boolean_p;

  Int i, j;
  UInt object_count[4];
  UInt index;


/*   if (1 < DmNumRecords(RecipeDB)) */

  /* 
   * FIRST:
   *   Load the 'sotu' record.
   *   Skip pointers.
   */
  index = 10; /* 0-9 are Top-10 records */
  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    load_sotu = (struct state_of_the_union *) p;

    sotu->being_held = load_sotu->being_held;
    sotu->see_invisible = load_sotu->see_invisible;
    sotu->detect_monster = load_sotu->detect_monster;
    sotu->wizard = load_sotu->wizard;
    sotu->score_only = load_sotu->score_only;

    StrNCopy(sotu->hunger_str, load_sotu->hunger_str, 8);
    StrNCopy(sotu->username, load_sotu->username, 32);

    sotu->cur_level = load_sotu->cur_level;
    sotu->max_level = load_sotu->max_level;
    sotu->cur_room = load_sotu->cur_room;
    sotu->party_room = load_sotu->party_room;
    sotu->r_de = load_sotu->r_de;
    sotu->m_moves = load_sotu->m_moves;
    sotu->foods = load_sotu->foods;
    sotu->party_counter = load_sotu->party_counter;
    sotu->bear_trap = load_sotu->bear_trap;
    sotu->halluc = load_sotu->halluc;
    sotu->blind = load_sotu->blind;
    sotu->confused = load_sotu->confused;
    sotu->levitate = load_sotu->levitate;
    sotu->haste_self = load_sotu->haste_self;
    //    sotu->fruit_number = load_sotu->fruit_number;

    sotu->warning_level = load_sotu->warning_level;
    sotu->warning_moves = load_sotu->warning_moves;

    /*
      // instead, we're recalculating these by calling ring_stats
    sotu->ring_flags = load_sotu->ring_flags;
    sotu->r_rings = load_sotu->r_rings;
    sotu->e_rings = load_sotu->e_rings;
    sotu->stealthy = load_sotu->stealthy;
    sotu->add_strength = load_sotu->add_strength;
    sotu->regeneration = load_sotu->regeneration;
    sotu->ring_dex = load_sotu->ring_dex;
    sotu->auto_search = load_sotu->auto_search;
    */
    //    init_fruit();
  }
  MemHandleUnlock(vh);

  /* 
   * SECOND:
   *   Load the 'rogue' record.
   *   Skip pointers.
   */

  index += 1;
  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    load_rogue = (fighter *) p;
    
    sotu->roguep->hp_current = load_rogue->hp_current;
    sotu->roguep->hp_max = load_rogue->hp_max;
    sotu->roguep->str_current = load_rogue->str_current;
    sotu->roguep->str_max = load_rogue->str_max;
    sotu->roguep->gold = load_rogue->gold;
    sotu->roguep->exp = load_rogue->exp;
    sotu->roguep->exp_points = load_rogue->exp_points;
    sotu->roguep->row = load_rogue->row;
    sotu->roguep->col = load_rogue->col;
    sotu->roguep->fchar = load_rogue->fchar;
    sotu->roguep->moves_left = load_rogue->moves_left;
    /* go through inventory to find armor, weapon, left/right rings */
  }
  MemHandleUnlock(vh);

  /* 
   * THIRD:
   *   Load the 'dungeon' array.
   */

  index += 1;
  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    load_dungeon = (UShort *) p;
    for (i = 0 ; i < DROWS ; i++) {
      for (j = 0 ; j < DCOLS ; j++) {
	sotu->dungeon[i][j] = *load_dungeon;
	load_dungeon++;
      }
    }
  }
  MemHandleUnlock(vh);

  /* 
   * THIRD AND A HALF:
   *   Load the 'buffer' array - this will be colossally slow
   */

  index += 1;
  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    load_buf = (Char *) p;
    for (i = 0 ; i < DROWS ; i++) {
      for (j = 0 ; j < DCOLS ; j++) {
	mvaddch2(i, j, *load_buf);
	load_buf++;
      }
    }
  }
  MemHandleUnlock(vh);

  /* 
   * FOURTH:
   *   Load the 'rooms' array.
   */

  index += 1;
  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    load_room = (room *) p;
    for (i = 0 ; i < MAXROOMS ; i++) {
      MemMove(&(sotu->rooms[i]), load_room, sizeof(room)); /* works? */
      load_room++;
    }
  }
  MemHandleUnlock(vh);

  /* 
   * FIFTH:
   *   Load the 'traps' array.
   */

  index += 1;
  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    load_trap = (trap *) p;
    for (i = 0 ; i < MAX_TRAPS ; i++) {
      MemMove(&(sotu->traps[i]), load_trap, sizeof(trap)); /* works! */
      load_trap++;
    }
  }
  MemHandleUnlock(vh);

  /* 
   * SIXTH:
   *   Load the 'id_' arrays.
   */
  load_character_id(sotu->id_scrolls, SCROLLS, ++index);
  load_character_id(sotu->id_potions, POTIONS, ++index);
  load_character_id(sotu->id_wands,   WANDS,   ++index);
  load_character_id(sotu->id_rings,   RINGS,   ++index);
  //  load_character_id(sotu->id_weapons, WEAPONS, ++index);
  //  load_character_id(sotu->id_armors,  ARMORS,  ++index);

  /* 
   * SIXTH and a half
   *   Load the 'is_wood' array... along with object-count info ...
   */

  index += 1;
  vh = DmQueryRecord(RogueDB, index); /* read-only, promise */
  p = MemHandleLock(vh);
  if (p) {
    uint_p = (UInt *) p;
    for (i = 0 ; i < 4 ; i++) {
      object_count[i] = uint_p[i];
    }
    uint_p +=4 ;
    boolean_p = (Boolean *) uint_p;
    for (i = 0 ; i < WANDS ; i++) {
      sotu->is_wood[i] = boolean_p[i];
    }
  }
  MemHandleUnlock(vh);

  /* 
   * SEVENTH:
   *   Load the 'rogue->pack', 'level_monsters', and 'level_objects'
   *   Yowza!  If this works I will be SHOCKED.
   */

  index += 1; /* it was up to 12 */
  load_character_objs(&(sotu->roguep->pack), &index, object_count[0]);
  fix_verbed_objs(&(sotu->roguep->pack), sotu->roguep);
  load_character_objs(sotu->level_monsters,  &index, object_count[1]);
  load_character_objs(sotu->level_objects,   &index, object_count[2]);

  /* I could use 'index', but I'm distrustful. */
  index = DmNumRecords(RogueDB);
  for (i = 10 ; i < index ; i++) {
    DmRemoveRecord(RogueDB, 10); /* check for error? */
    /* Do not remove records #0-9. They are the Top N list. */
  }

  /* let's see if this will fly..  not bad. */
  // the following has moved to iRogue.c:
  /*
  move_visible_window(sotu->roguep->col, sotu->roguep->row, true);
  if (sotu->cur_room != PASSAGE)
    light_up_room(sotu->cur_room, sotu);
  else {
    light_passage(sotu->roguep->row, sotu->roguep->col, sotu);
    mvaddch(sotu->roguep->row, sotu->roguep->col, sotu->roguep->fchar);
  }
  print_stats(STAT_ALL, sotu);
  */
}

/**********************************************************************
                       UPDATE_FIELD_SCROLLERS
 IN:
 frm, fld, up_scroller, down_scroller = various UI doodads
 OUT:
 nothing
 PURPOSE:
 Update the given scroller widgets (for the given field 
 (in the given form)), according to whether the field is scrollable
 in the "up" and "down" directions.
 **********************************************************************/
void update_field_scrollers(FormPtr frm, FieldPtr fld,
			    Word up_scroller, Word down_scroller) 
{
  Boolean u, d;
  u = FldScrollable(fld, up);
  d = FldScrollable(fld, down);
  FrmUpdateScrollers(frm, 
		     FrmGetObjectIndex(frm, up_scroller),
		     FrmGetObjectIndex(frm, down_scroller),
		     u, d);
  return;
}


/**********************************************************************
                       PAGE_SCROLL_FIELD
 IN:
 frm, fld = various UI doodads
 dir = whether to scroll 'fld' up or down
 OUT:
 PURPOSE:
 Call this to scroll the field 'fld' up/down by one "page".
 (The caller should call update_field_scrollers immediately afterward.)
 **********************************************************************/
void page_scroll_field(FormPtr frm, FieldPtr fld, DirectionType dir)
{
  Word linesToScroll;

  /* how many lines can we scroll? */
  if (FldScrollable(fld, dir)) {
    linesToScroll = FldGetVisibleLines(fld) - 1;
    FldScrollField(fld, linesToScroll, dir);
  }

  return;
}

/**********************************************************************
                       GET_USERNAME
 IN:
 sotu = various globals (username)
 OUT:
 nothing
 PURPOSE:
 Get a default username, truncating it if necessary.
 (originally from Reptoids, modified some.)
 **********************************************************************/
void get_username(struct state_of_the_union *sotu) {
  Char *tmp, *first_wspace;
  VoidHand h;
  tmp = md_malloc2(sizeof(Char) * (dlkMaxUserNameLength + 1));
  DlkGetSyncInfo(NULL, NULL, NULL, tmp, NULL, NULL);
  /* if it's too long, use the first name only */
  if (StrLen(tmp) > 30) {
    first_wspace = StrChr(tmp, spaceChr);
    if (first_wspace)
      *(first_wspace) = '\0';
    else
      tmp[30] = '\0';
  }
  /* username is 32 chars allocated */
  if (StrLen(tmp))
    StrNCopy(sotu->username, tmp, 31);
  else
    StrPrintF(sotu->username, "rogue");

  h = MemPtrRecoverHandle(tmp);
  if (h) MemHandleFree(h);  
}


/*
enum patterns { blackPattern, whitePattern, grayPattern, customPattern };
typedef enum patterns PatternType;
typedef Word CustomPatternType [4];
*/
/*static CustomPatternType bombPattern = { 0x8855, 0x2255, 0x8855, 0x2255 };*/
/*static CustomPatternType graePattern = { 0xaa55, 0xaa55, 0xaa55, 0xaa55 };*/
/*static CustomPatternType diagPattern = { 0x8844, 0x2211, 0x8844, 0x2211 };*/

/**********************************************************************
                       SHOW_DUNGEON
 IN:
 sotu = various globals (dungeon, rooms, wizard, cur_level, rogue)
 OUT:
 nothing
 PURPOSE:
 The "real" dungeon is 24 wide x 80 high, and doesn't all fit on the
 (156^2 pixel) screen at once when it's "full size".  So I have a
 form to display a tiny version of the map (just rooms/mazes/tunnels,
 the rogue, and the stairs), where each square is 2 x 3 pixels.
 Maybe I could put flyspecks for monsters/objects too.  Maybe.
 **********************************************************************/
//static CustomPatternType yourPattern = { 0xaaaa, 0x5555, 0xaaaa, 0x5555 };
// The passage pattern got messed up so I moved it inside.
void show_dungeon(struct state_of_the_union * sotu) {
  RectangleType r;
  Word col, row, i, w, h;
  UShort mask;
  Char c;
  CustomPatternType origPattern;
  CustomPatternType yourPattern = { 0xaaaa, 0x5555, 0xaaaa, 0x5555 };
  UShort ** dungeon = sotu->dungeon;
  room * rooms = sotu->rooms;

  RctSetRectangle(&r, 0,0, 160, 160);
  WinDrawRectangle(&r, 6);

  /* This is just some fluff to put in the extra space above and below. 
     I could probably think of some more-useful fluff to put there. */
  if (!sotu->wizard) {
    if (sotu->cur_level % 2 == 1) {
      WinDrawInvertedChars("You Can't Get There From Here", 29, 15, 1);
      WinDrawInvertedChars("(Tap  Anywhere  When  Done)", 27, 22, 147);
    } else {
      WinDrawInvertedChars("Strange, This Doesn't Look Like ...", 35, 12, 1);
      WinDrawInvertedChars("Oops, I Was Holding It Upside Down.", 35, 8, 147);
    }
  } else {
    WinDrawInvertedChars("Here  There  Be  Dragons", 24, 32, 1);
    WinDrawInvertedChars("(Nothing  You  Can't  Handle,  Eh?)", 35, 12, 147);
  }

  /* This is just some decoration. */
  WinEraseLine(8, 34, 148, 34);
  WinEraseLine(6, 36, 150, 36);
  WinEraseLine(4, 38, 152, 38);
  
  /* Each tile gets 2 pixels/col and 3 pixels/row. */
  RctSetRectangle(&r, 0,42, 160,72); /* left,top, width,height */
  WinEraseRectangle(&r, 0);
  WinDrawRectangleFrame(simpleFrame, &r); /* simple,round,popup,boldRound */

  /* Draw the rooms of the dungeon as rectangles */
  for (i = 0 ; i < MAXROOMS ; i++) {
    if (rooms[i].is_room & R_ROOM) {
      col = rooms[i].left_col;
      row = rooms[i].top_row;
      c = peek_me2(row, col); /* this is a gross hack */
      if ( sotu->wizard || (c && c != ' ') ) {
	w = rooms[i].right_col - col;
	h = rooms[i].bottom_row - row;
	/* col * 2, row * 3 + 42, w * 2, h * 3... -1 all round. */
	RctSetRectangle(&r, col * 2 + 2, row * 3 + 42 + 2, 
			    w * 2 - 2,   h * 3 - 1);
	WinDrawRectangleFrame(simpleFrame, &r);
      }
    }
  }
  /* Draw the rest of the dungeon square by square */
  for (col = 0 ; col < DCOLS ; col++) {
    for (row = 0 ; row < DROWS ; row++) {
      RctSetRectangle(&r, col*2, row*3 + 42, 2,3);
      c = peek_me2(row, col); /* this is a gross hack */
      if ( sotu->wizard || (c && c != ' ') ) {
	mask = dungeon[row][col];
	if ( (mask & STAIRS) || (mask & TUNNEL) ) {
	  WinGetPattern(origPattern);
	  WinSetPattern(yourPattern);
	  WinFillRectangle(&r, 0);
	  WinSetPattern(origPattern);
	} else if (mask & FLOOR) {
	  /* 	WinDrawRectangle(&r, 0); */
	} else if (0 && (mask & HORWALL) ) {
	  WinDrawLine(col*2 + 1, row*3 + 42 + 1,
		      col*2 + 3, row*3 + 42 + 1);
	} else if (0 && (mask & VERTWALL) ) {
	  WinDrawLine(col*2 + 1, row*3 + 42 + 1,
		      col*2 + 1, row*3 + 42 + 4);
	} else if (mask & DOOR) {
	  if (! (mask & HIDDEN)) {
	    WinInvertRectangle(&r, 0);
	  }
	}
      }
      /* used to have "else" (peekme==' ') "erase rectangle" */
    } /* end for row */
  } /* end for col */

  /* don't forget to put the rogue somewhere */
  col = sotu->roguep->col * 2 - 1;
  if (col < 0) col = 0;
  row = sotu->roguep->row * 3 + 42 - 1;
  RctSetRectangle(&r, col, row, 
		      2+1, 3+1);
  WinDrawRectangle(&r, 0);
  return;
}


/**********************************************************************
                       DO_FEEP
 IN:
 frq = frequency
 dur = duration
 OUT:
 nothing
 PURPOSE:
 This will produce a beep of the requested frequency and duration,
 if the pilot owner has "game sound" turned on in the Preferences app.
 (Try to keep it down to chirps...)
 **********************************************************************/
/* SoundLevelType: slOn, slOff
   gameSoundLevel found in SystemPreferencesType
 */
void do_feep(Long frq, UInt dur) {
  /* prefGameSoundLevelV20? */
  if (PrefGetPreference(prefGameSoundLevel) != slOff) {
    /* click: 200, 9
       confirmation: 500, 70  */
    SndCommandType sndCmd;
    sndCmd.cmd = sndCmdFreqDurationAmp; /* "play a sound" */
    sndCmd.param1 = frq; /* frequency in Hz */
    sndCmd.param2 = dur; /* duration in milliseconds */
    sndCmd.param3 = sndDefaultAmp; /* amplitude (0 to sndMaxAmp) */
    SndDoCmd(0, &sndCmd, true);
  }
}

/**********************************************************************
                       DRAW_CIRCLE
 IN:
 x,y = center (pixels)
 radius = radius (pixels)
 b = whether to draw it in black (or erase it in "white")
 various globals
 OUT:
 nothing
 PURPOSE:
 Draw a circle on the screen...
 **********************************************************************/
void draw_circle(Short x, Short y, Short radius, Boolean b) {
  RectangleType r;
  RctSetRectangle(&r, x-radius, y-radius, 2*radius, 2*radius);
  if (b)
    WinDrawRectangle(&r, radius);
  else 
    WinEraseRectangle(&r, radius);
}


/**********************************************************************
                       TOGGLE_RING_BUTTONS
 IN:
 frm = the form that has these buttons in it
 hide = whether to hide or display the buttons
 remove = whether the player wants to 'remove' or 'put on' a ring
 various globals
 OUT:
 nothing
 PURPOSE:
 There is a form that allows the user to perform actions on an item
 selected from an inventory list.  If the action selected is related
 to rings, the buttons for 'left' and 'right' should be displayed
 so the user can select which hand to put the ring on; otherwise
 they should be hidden.  If the action selected is to remove a ring,
 a hand's button should be displayed iff there's a ring on that hand;
 if the action selected is to put on a ring, display a hand's button
 only if there's no ring on that hand.
 **********************************************************************/
/**********************************************************
****** HEISENBUG - null string passed to stringmgr ********
**************** fixed now? *******************************
***********************************************************/
void toggle_ring_buttons(FormPtr frm, Boolean hide,
			 struct state_of_the_union *sotu)
{
  static Boolean ring_buttons_visible = true;
  ControlPtr pushbtn;
  Short i;
  Boolean set_me = true;
  if (!hide) {
    // Re-show buttons even if they're (unlikely) already visible
    ring_buttons_visible = true;
    for (i = pbtn_i_left1 ; i <= pbtn_i_right4 ; i++) {
      pushbtn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, i));
      // Show only buttons corresponding to ring-free fingers ...
      if (!sotu->roguep->rings[i-pbtn_i_left1]) {
	CtlShowControl(pushbtn);
	CtlSetValue(pushbtn, set_me);
	set_me = false;
      } else {
	CtlHideControl(pushbtn);
	CtlSetValue(pushbtn, 0);
      }
    }
  } else if (ring_buttons_visible) {
    // Don't bother hiding the buttons if they're already hidden.
    ring_buttons_visible = false;
    for (i = pbtn_i_left1 ; i <= pbtn_i_right4 ; i++) {
      pushbtn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, i));
      CtlHideControl(pushbtn);
      CtlSetValue(pushbtn, 0);
    }
  } 
}

/**********************************************************************
                       get_ring_finger
 IN:
 inventory form
 various globals
 OUT:
 what ring finger is currently selected
 PURPOSE:
 called when the player wants to put on a ring
 **********************************************************************/
// I could use bitwise shift but I might be shooting myself in the foot.
// oh, what the hell, let's try it. ("u=u<<1;" is the same as "u*=2;")
// yes, I realize this is grotesque even if it works
/*
ULong Old_get_ring_finger(FormPtr frm, struct state_of_the_union *sotu)
{
  ControlPtr pushbtn;
  Short i;
  UShort u; // i THINK this will be long enough...
  //   0000 0000 0000 0100 // left 1
  //   0000 0010 0000 0000 // right 1
  u = 4;
  for (i = pbtn_i_left1 ; i <= pbtn_i_right4 ; i++) {
    pushbtn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, i));
    if (CtlGetValue(pushbtn))
      return (ULong) u;
    u = u << 1;
  }
  return 0;
}
*/
Short get_ring_finger(FormPtr frm, struct state_of_the_union *sotu)
{
  ControlPtr pushbtn;
  Short i;
  for (i = pbtn_i_left1 ; i <= pbtn_i_right4 ; i++) {
    pushbtn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, i));
    if (CtlGetValue(pushbtn))
      return (i - pbtn_i_left1);
  }
  return -1;
}

/**********************************************************************
                       DEDUCE_TASK
 IN:
 index of inventory-object currently selected
 return value -- primary DWIM-task
 return value -- secondary DWIM-task
 various globals
 OUT:
 nothing
 PURPOSE:
 Called when an inventory object is selected; it figures out what
 tasks are appropriate to it.
 **********************************************************************/

static void deduce_task(Short index, Short *primary, Short *secondary,
			struct state_of_the_union *sotu)
{
  object *obj = get_nth_object2(index, &(sotu->roguep->pack));
  ErrNonFatalDisplayIf ( !obj,
			 "XXXXXX Something Is Very Wrong");  // serious error.
  *primary = 0;  // 'primary' is the all-purpose button
  *secondary = (obj->in_use_flags & BEING_WIELDED) ? INV_UNWIELD : INV_WIELD;
  switch (obj->what_is) {
  case FOOD:
    *primary = INV_EAT;
    break;
  case POTION:
    *primary = INV_QUAFF;
    break;
  case SCROLL:
    *primary = INV_READ;
    break;
  case WAND:
    *primary = INV_ZAP;
    break;
  case RING:
    *secondary = 0; // can't wield a ring.
    if (obj->in_use_flags & ON_EITHER_HAND)
      *primary = INV_REMOVE;
    else if (sotu->r_rings < 8)
      *primary = INV_PUTON; // there are still fingers available
    break;
  case ARMOR:
    *secondary = 0; // can't wield armor.
    *primary = (obj->in_use_flags & BEING_WORN) ? INV_TAKEOFF : INV_WEAR;
    // (if an armor is already on, you will spend an extra turn taking it off)
    break;
  case WEAPON:
    *primary = *secondary;
    *secondary = (obj->o_flags & O_ZAPPED) ? INV_ZAP : 0;
    break;
  default:
    *primary = 0;
    *secondary = 0;
    break;
  }
}

/**********************************************************************
                       SPECIALIZE_LABELS
 IN:
 inventory-frobbing form
 index of inventory-object currently selected
 various globals
 OUT:
 nothing
 PURPOSE:
 Called when an inventory object is selected to set/hide/show the button
 labels to represent the set of tasks appropriate to that object.
**********************************************************************/
void specialize_labels(FormPtr frm, Short index,
		       struct state_of_the_union *sotu)
{
  Short first = 0;
  Short fourth = 0;
  ControlPtr btn;

  // Also, hide the ring-fingers pushbutton panel!!!
  // Reveal it IFF first==INV_PUTON.
  deduce_task(index, &first, &fourth, sotu);
  toggle_ring_buttons(frm, (first != INV_PUTON), sotu); // wheeee!
  // set or hide the first button
  btn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, pbtn_if_frob));
  CtlSetLabel(btn, specialized_labels[first]);
  if (first) {
    CtlShowControl(btn);    
    CtlSetValue(btn, 1); // to turn it on...
  }
  else
    CtlHideControl(btn);
  // set or hide the fourth button
  btn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, pbtn_if_wield));
  CtlSetLabel(btn, specialized_labels[fourth]);
  if (fourth)
    CtlShowControl(btn);    
  else
    CtlHideControl(btn);
  // I need some better way to refresh the lines between them!!!
  btn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, pbtn_if_drop));
  CtlShowControl(btn);
  btn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, pbtn_if_throw));
  CtlShowControl(btn);
  // Hey, maybe I should do something clever,
  // like have it default to "first button ON if we're "show"ing it"
  // This might deal well with the common case.
  // ....later.....

}

/**********************************************************************
                       DO_WHAT_I_MEAN
 IN:
 index of inventory-object currently selected
 index of button currently selected
 various globals
 OUT:
 what "I mean"
 PURPOSE:
 Called when the player taps "ok" in the inventory-frobbing dialog,
 this will return the task that should be performed on the object.
**********************************************************************/
Short do_what_i_mean(Short index, Short button,
		    struct state_of_the_union *sotu)
{
  Short first = 0;
  Short fourth = 0;
  deduce_task(index, &first, &fourth, sotu);
  switch (button) {
  case pbtn_if_frob:
    return first;
  case pbtn_if_drop:
    return INV_DROP;
  case pbtn_if_throw:
    return INV_THROW;
  case pbtn_if_wield:
    return fourth;
  }
  return 0;
}
