/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/

#include "Pilot.h"
#include "Graffiti.h" /* just for GrfGetState */
#include "callback.h"
#include "iRogueRsc.h"
#include "Globals.h"

#include "rogue.h"

void explain_it2(Short x, Short y, struct state_of_the_union *sotu);
void update_field_scrollers2(FormPtr frm, FieldPtr fld,
			    Word up_scroller, Word down_scroller);
void page_scroll_field2(FormPtr frm, FieldPtr fld, DirectionType dir);

#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL

static Boolean ApplicationHandleEvent(EventPtr event);
static Boolean OpenDatabase(void);
static Word    StartApplication(void);
static void    StopApplication(void);
static void    EventLoop(void);

Short direction;
extern struct state_of_the_union * sotu;
Short x, y;
Short inventory_item;
Short task;

Short run_walk_border = 4;
Short walk_search_border = 1;

Boolean run_on = true;
Boolean to_the_biteme = false;
Boolean dave_mode = false;
Boolean scroll_mode = false;

Boolean save_on_exit = true;

Boolean inventory_not_messages = true;

Boolean do_make_new_level = false;
Boolean save_me = true;
Boolean i_am_dead = false;

Short count = 0;
Boolean whatsit_p = false;

/* define INV_foo MOVED to rogue_defines.h */

#define abs(a)                                  (((a) >= 0) ? (a) : (-(a)))
#define sgn(x) ( (x) >= 0 ? 1 : -1 )

/* needed these from lib_curses */
/* The screen can have 15 rows, but I took 4 for messages and stats */
#define visible_h 11
/* You can put about 19.9 m's on a memo line.. so.. we'll have 20 cols */
#define visible_w 20

/*****************************************************************************
 *                                                                           *
 *                      Being Tested                                         *
 *                                                                           *
 *****************************************************************************/
void start_of_Play_Level_loop() {

  sotu->interrupted = false;
  /*
  if (hit_message[0]) {
    message(hit_message, 1);
    hit_message[0] = 0;
  }
  */

  if (sotu->trap_door) {
    /* not sure about this - from use of '>' */
    free_old_level(sotu);
    do_feep(300, 9);
    make_new_level();
    move_visible_window(rogue->col, rogue->row, true);
    /* end not sure */
    sotu->trap_door = 0;
    return;
  }
/*    move(rogue->row, rogue->col); */
  refresh();

}

/*******************************************************************
 * Caller had darn well better check 'c' is a reasonable character *
 * c = e->data.keyDown.chr    OR    c = e->data.menu.itemID - 4000 *
 *                  Returns:  "handled"                            */
static Boolean main_help_handle_cmd(Char c) {
  FormPtr frm;
  to_the_biteme = false;
  frm = FrmGetActiveForm();

  check_message();

  /* Implemented:    m   d s i f F e q r z w t P R T W < >
     Not impl.:    .   ,   (requires rest(), kick_into_pack())
           also,   c
   */
  /* Removed:        d         e q r   w   P R
   */

  switch(c) {
  case '>':
  case pageDownChr:
    if (drop_check(sotu)) {
      free_old_level(sotu);
      do_feep(300, 9);
      make_new_level();
      move_visible_window(rogue->col, rogue->row, true);
    }
    return true;
  case '<':
  case pageUpChr:
    if (check_up()) {
      free_old_level(sotu);
      do_feep(400, 9);
      make_new_level();
      move_visible_window(rogue->col, rogue->row, true);
    }
    return true;
  case 'm':
    direction = NO_DIRECTION;
    task = INV_MOVE;
    FrmPopupForm(DirectionForm);
    return true;
  case 'F':
    to_the_biteme = true;
    /* FALL THROUGH to 'f' !!! */
  case 'f':
    direction = NO_DIRECTION;
    task = INV_FIGHT;
    FrmPopupForm(DirectionForm);
    return true;
    /*
  case 'l':
    // List/Long inventory
    if (rogue->pack.next_object != NULL) {
      inventory_not_messages = true;
      FrmPopupForm(InvViewForm);
    } else {
      message("your pack is empty\n", sotu);
    }
    return true;
    */
  case 'i':
    /* Inventory */
    if (rogue->pack.next_object != NULL) {
      inventory_item = -1;
      task = INV_DROP;
      FrmPopupForm(InvSelectForm);
    } else {
      message("your pack is empty\n", sotu);
    }
    return true;
    /*
      case 'c':
      inventory_item = -1;
      task = INV_CALL;
      FrmPopupForm(InvSelectForm);
      return true;
      */
  case 's':
    search( ((count > 0) ? count : 1), false);
    count = 0;
    /*    search(1, false); */
    return true;
  case '.':
    rest( (count > 0) ? count : 1 );
    count = 0;
    /*    rest(1); */
    return true;
  case ',':
    kick_into_pack();
    return true;
  case 'Q':
    /* do an Alert-thingy to Confirm, then:
    killed_by((object *) 0, QUIT);
    */
    if (0 == FrmAlert(QuitP)) {
      killed_by((object *) 0, QUIT, sotu);
      FrmPopupForm(TopTenForm);
    }
    return true;
  case '^':
    direction = NO_DIRECTION;
    task = INV_IDTRAP;
    FrmPopupForm(DirectionForm);
    return true;
  case 't':
    /* These two are NOT in the menu */
    inventory_item = -1;
    direction = NO_DIRECTION;
    task = INV_THROW;
    FrmPopupForm(DirectionForm);
    return false;
  case 'z':
    inventory_item = -1;
    direction = NO_DIRECTION;
    task = INV_ZAP;
    FrmPopupForm(DirectionForm);
    return false;
  }
  return false;
}

Boolean Main_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
/*   Boolean last_was_digit; */
  Short tmp_x, tmp_y, click_dir;
  Boolean do_run = true;
  Char buf[40];

    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    /* mine */
    /** place any form-initialization stuff here **/
    /** init_game(); should go somewhere ELSE. */
    /* b/c this stuff will be called when we return from a popup. ? */
    /* end mine */	
    FrmDrawForm(frm);

    if (do_make_new_level) {
      do_make_new_level = false;
      make_new_level();
    } else {
      /*       message ("Hey!  I found a database.", sotu); */
      clear_level(sotu);
      load_character(sotu);
      ring_stats(true, sotu); // recalculate some calculated parts of sotu
      /* let's see if this will fly..  not bad. */
      move_visible_window(sotu->roguep->col, sotu->roguep->row, true);
      if (sotu->cur_room != PASSAGE)
	light_up_room(sotu->cur_room, sotu);
      else {
	light_passage(sotu->roguep->row, sotu->roguep->col, sotu);
	mvaddch(sotu->roguep->row, sotu->roguep->col, sotu->roguep->fchar);
      }
      print_stats(STAT_ALL, sotu);
      /* ok, done loading character! */
    }

    //    StrPrintF(buf, "Welcome to iRogue");
    // 171 = << 187 = >>
    // 249 is a pretty nice character.  also 165 and 167 (amuletish?)
    /*
    for (tmp_x = 165, tmp_y = 0 ; tmp_y < 32 ; ) {
      buf[tmp_y++] = tmp_x++;
      buf[tmp_y++] = ' ';
    }
    buf[tmp_y] = 0;
    message(buf, sotu);
    */
    message("Welcome to iRogue", sotu);
    start_of_Play_Level_loop();

    handled = true;
    goto SKIP_THE_REST;
    break;

  case menuEvent:
    MenuEraseStatus(NULL); /* clear menu status from display */
    // if there's a --more-- we need to show the next message
    whatsit_p = false;
    if (sotu->last_old_message_shown < SAVED_MSGS-1) {
      check_message();  // clear the old message
      message(0, sotu); // show the next buffered message
      //      message(0, sotu); // try to show a second one, but maybe fail
      handled = true;
      count = 0;
      goto SKIP_THE_REST;
    }
    switch(e->data.menu.itemID) {

    case menu_mainAbout:
      FrmPopupForm(AboutForm);
      handled = true;
      goto SKIP_THE_REST;
      //      break; // unnecessary
    case menu_mainMoveInstruct:
      FrmHelp(MoveStr);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainWiztog:
      sotu->wizard = !sotu->wizard;
      check_message(); 
      if (sotu->wizard)
	message("wizard on", sotu);
      else
	message("wizard off", sotu);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainWiz:
      check_message();
      if (sotu->wizard)
	FrmPopupForm(WizForm);
      else
	message("nice try", sotu);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainSettings:
      FrmPopupForm(BullseyeForm);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainGraffiti:
      SysGraffitiReferenceDialog(referenceDefault);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainWhatsit:
      check_message();
      message("tap on a character for clarification", sotu);
      whatsit_p = true;
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainMap:
      FrmPopupForm(MapForm);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainMsgs:
      inventory_not_messages = false;
      FrmPopupForm(InvViewForm);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    case menu_mainScroll:
      direction = NO_DIRECTION;
      task = SCROLL_WINDOW;
      FrmPopupForm(DirectionForm);
      handled = true;
      goto SKIP_THE_REST;
      //      break;
    default:
      /* this is a Monstrous Hack. */
      tmp_x = e->data.menu.itemID - MAGIC_MENU_NUMBER;
      if (tmp_x > 0 && tmp_x < 128)
	handled = main_help_handle_cmd((Char) tmp_x);
      /*      StrPrintF(buf, "you typed %c", e->data.menu.itemID - 4000);
	      message(buf, sotu); */
      count = 0;
    }
    handled = true;
    break;

  case keyDownEvent:
    // if there's a --more-- we need to show the next message
    whatsit_p = false;
    if (sotu->last_old_message_shown < SAVED_MSGS-1) {
      check_message();  // clear the old message
      message(0, sotu); // show the next buffered message
      //      message(0, sotu); // try to show a second one, but maybe fail
      handled = true;
      count = 0;
      goto SKIP_THE_REST;
    }
    /* graffiti or up/down hardware button. */
    /* I farmed this out so that the Monstrous Hack menu can use it too. */    
    handled = main_help_handle_cmd(e->data.keyDown.chr);
    if (handled)
      count = 0;
    else if ('0' <= e->data.keyDown.chr
	     && e->data.keyDown.chr <= '9') {
      if (count <= 0) {
	count = e->data.keyDown.chr - '0';
      } else if (count <= 9) {
	count = count * 10 + e->data.keyDown.chr - '0';
      } else {
	count = 99; /* maxed-out */
      } // xxx Hey, I need to make "backspace" work too.
      check_message();
      StrPrintF(buf, "count: %d", count);
      message(buf, sotu);
      handled = true;
      goto SKIP_THE_REST;
    }
    break;

  case penDownEvent:
    // if there's a --more-- we need to show the next message
    if (sotu->last_old_message_shown < SAVED_MSGS-1) {
      check_message();  // clear the old message
      message(0, sotu); // show the next buffered message
      //      message(0, sotu); // try to show a second one, but maybe fail
      handled = true;
      count = 0;
      goto SKIP_THE_REST;
    }
    if (whatsit_p) {
      explain_it2(e->screenX, e->screenY, sotu);
      whatsit_p = false;
      goto SKIP_THE_REST;
    }
    check_message();
    tmp_x = e->screenX - 80;
    tmp_y = e->screenY - 80;
    if ((abs(tmp_x) < walk_search_border*10) && 
	(abs(tmp_y) < walk_search_border*10)) {
      /*       message("searching", sotu);  */
      do_feep(200,9);
      search( ((count > 0) ? count : 1), false);
      handled = true;
      count = 0;
      break;
    } else if ((abs(tmp_x) < run_walk_border*10) &&
	       (abs(tmp_y) < run_walk_border*10)) {
      do_run = false;
    }
    if ((abs(tmp_x)) > 2 * (abs(tmp_y))) {
      /* mostly x, move in the x direction */
      click_dir = tmp_x > 0 ? EAST : WEST;
    } else if ((abs(tmp_y)) > 2 * (abs(tmp_x))) {
      /* mostly y */
      click_dir = tmp_y > 0 ? SOUTH : NORTH;
    } else {
      /* closer to diagonal */
      if (tmp_y > 0) {
	/* southish */
	click_dir = tmp_x > 0 ? SOUTHEAST : SOUTHWEST;
      } else {
	/* northish */
	click_dir = tmp_x > 0 ? NORTHEAST : NORTHWEST;	
      }
    }
    if (scroll_mode && (abs(tmp_x) > 70 || abs(tmp_y) > 70)) {
      /* instead of moving, scroll the screen. */
      scroll_window(click_dir, max(1, count));
      refresh();
      handled = true;
      count = 0;
      goto SKIP_THE_REST;
    } else if (run_on && do_run) {
      multiple_move_rogue_Ctrl(click_dir, dave_mode); /*stop when interesting*/
      do_feep(200,9);
    } else {
      one_move_rogue(click_dir, true); /* tap tap tap tap */
      do_feep(200,9);
    }
    handled = true;
    count = 0;
    break;

  default:
    break;
  }
/*   last_was_digit = false; */

  start_of_Play_Level_loop();
  check_hit_message();

  /* don't do anything after this point! */
SKIP_THE_REST:

    CALLBACK_EPILOGUE

  return handled;
}

/* how to exit a popup form that you can enter from more than one place */
extern void LeaveForm  ()
{
   FormPtr frm;
   frm = FrmGetActiveForm();
   FrmEraseForm (frm);
   FrmDeleteForm (frm);
   FrmSetActiveForm (FrmGetFirstForm ());
}


/**********************************************************************
                       INIT_DIRECTION_SELECT_LIST
 IN:
 frm = the direction select form
 zap = whether to initialize a list for 'throw' or for 'zap'
 various globals
 OUT:
 nothing
 PURPOSE:
 This is called when the direction select form is initialized.
 It initializes the popup list with inventory items that are
 appropriate for "zapping" or "throwing".
 **********************************************************************/
Char ** direction_select_items;
Short direction_select_numItems;
void make_it_fit(CharPtr buf, Short avail_space, Short maxbuflen);
static void init_direction_select_list(FormPtr frm, Boolean zap) {
  ListPtr lst; // popup list: list_d
  ControlPtr ctl; // popup trigger: popup_d
  CharPtr label;
  Char buf[80];
  object *obj;
  Word i, numItems = 0;
  Char cur_ichar = 'a';

  for (obj = rogue->pack.next_object ; obj != NULL ; obj = obj->next_object) {
    obj->ichar = cur_ichar++; /* re-order the inventory letters */
    if (cur_ichar > 'z')
      cur_ichar = 'A';
    if ( (zap && ((obj->what_is & WAND)
		  || ((obj->what_is & WEAPON)
		      && (obj->o_flags & O_ZAPPED))))
	 || ((!zap && (obj->what_is & WEAPON)
	      && (obj->o_flags & O_MISSILE)))  ) {
      numItems++;
    }
  }
  /* Hey!  Who's freeing this puppy? */
  direction_select_items = (Char **) md_malloc(sizeof(Char *) * numItems);
  for (obj = rogue->pack.next_object, i = 0 ;
       (obj != NULL && i < numItems) ;
       obj = obj->next_object) {
    if (   (zap  && ((obj->what_is & WAND)
		  || ((obj->what_is & WEAPON)
		      && (obj->o_flags & O_ZAPPED))))
	   || ((!zap && (obj->what_is & WEAPON)
		 && (obj->o_flags & O_MISSILE)))  ) {
      buf[0] = obj->ichar; 
      buf[1] = ')';
      buf[2] = ' ';
      get_desc_brief(obj, buf+3, sotu); /* er.  will it overflow 80 chars? */
      /* Insert inventory code to truncate it to a reasonable length? */
      make_it_fit(buf, 140, 80);
      direction_select_items[i] = md_malloc(sizeof(Char)*(StrLen(buf)+1));
      // malloc failed
      ErrNonFatalDisplayIf ( (direction_select_items[i] == NULL),
			     "XXXXXX SERIOUS EVIL");
      StrCopy(direction_select_items[i], buf);

      i++;
    }
  }

  lst = FrmGetObjectPtr (frm, FrmGetObjectIndex(frm, list_d));
  LstSetListChoices(lst, direction_select_items, numItems);
  if (numItems > 0) {
    if (numItems > 10) numItems = 10;
    LstSetHeight(lst, numItems);
  }
  LstSetSelection (lst, 0);
  ctl = FrmGetObjectPtr (frm, FrmGetObjectIndex(frm, popup_d));
  label = LstGetSelectionText (lst, 0);
  if (label) {
    if (label[0] > 'Z')
      inventory_item = label[0] - 'a';
    else
      inventory_item = label[0] - 'A';
  }
  CtlSetLabel(ctl, label);
  LstSetSelection(lst, 0);
}
static void init_direction_select(FormPtr frm, Short task) {
/*   ControlPtr btn; */
  ControlPtr ctl; // popup trigger: popup_d.  Hide it by default.
  ctl = FrmGetObjectPtr (frm, FrmGetObjectIndex(frm, popup_d));
  CtlHideControl(ctl);
  switch(task) {
  case INV_ZAP:
    FrmCopyTitle(frm, "zap:  direction?");
    /* example of changing a label: */
    /*     btn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, btn_d_cancel)); */
    /*     CtlSetLabel(btn, "NO ZAP"); */
    if (inventory_item == -1)
      init_direction_select_list(frm, true);
    break;
  case INV_THROW:
    FrmCopyTitle(frm, "throw:  direction?");
    if (inventory_item == -1)
      init_direction_select_list(frm, false);
    break;
  case INV_MOVE:
    FrmCopyTitle(frm, "move:  direction?");
    break;
  case INV_FIGHT:
    FrmCopyTitle(frm, "fight:  direction?");
    break;
  case INV_IDTRAP:
    FrmCopyTitle(frm, "i.d. trap:  direction?");
    break;
  case SCROLL_WINDOW:
    FrmCopyTitle(frm, "scroll view:  direction?");
    break;
  }
}
Boolean Direction_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
  ListPtr lst;
  ControlPtr ctl;
  CharPtr txt;
  static Boolean pick_item = false;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    /* mine */
    /** place any form-initialization stuff here **/
    pick_item = (inventory_item == -1) ? true : false;
    init_direction_select(frm, task);
    /* end mine */	
    FrmDrawForm(frm);
    if ((task == INV_ZAP || task == INV_THROW) && pick_item) {
      ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, popup_d));
      CtlShowControl(ctl);
    }
    handled = true;
    break;

  case popSelectEvent:
    frm = FrmGetActiveForm();
    if (e->data.popSelect.listID == list_d) {
      if (pick_item && (task == INV_ZAP || task == INV_THROW)) {
	lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, list_d));
	txt = LstGetSelectionText (lst, e->data.popSelect.selection);
	if (txt[0]) {
	  inventory_item = txt[0] - 'a'; // dangerous
	} else {
	  inventory_item = -1;
	}
      }
    }
    /*      handled = true; */
    break;

  case ctlSelectEvent:
    direction = -1;
    switch(e->data.ctlSelect.controlID) {
    case btn_d_n:
      direction = NORTH;
      break;
    case btn_d_ne:
      direction = NORTHEAST;
      break;
    case btn_d_e:
      direction = EAST;
      break;
    case btn_d_se:
      direction = SOUTHEAST;
      break;
    case btn_d_s:
      direction = SOUTH;
      break;
    case btn_d_sw:
      direction = SOUTHWEST;
      break;
    case btn_d_w:
      direction = WEST;
      break;
    case btn_d_nw:
      direction = NORTHWEST;
      break;
    case btn_d_cancel:
      direction = NO_DIRECTION;
      break;
    default:
      break;
    }
    if (direction != -1) {
      LeaveForm();
      if (direction != NO_DIRECTION) {
	switch(task) {
	  /*
	    case INV_ZAP:
	    case INV_THROW:
	    inventory_item = -1;
	    FrmPopupForm(InvSelectForm);
	    break;
	  */
	case INV_ZAP:
	  /* inventory_item should be set already */
	  if (inventory_item != -1) {
	    if (zapp(direction, inventory_item, sotu))
	      reg_move();
	  }
	  break;
	case INV_THROW:
	  /* inventory_item should be set already */
	  if (inventory_item != -1) {
	    if (throw(direction, inventory_item, sotu))
	      reg_move();
	  }
	  break;
	case INV_MOVE:
	  /* replacing move_onto() */
	  one_move_rogue(direction, false);
	  break;
	case INV_FIGHT:
	  fight(direction, to_the_biteme); /* true = to-the-death */
	  break;
	case INV_IDTRAP:
	  id_trap(direction, sotu);
	  break;
	case SCROLL_WINDOW:
	  scroll_window(direction, max(1, count));
	  count = 0;
	  break;
	}
      }
      handled = true;
    }
    break;

  default:
    break;
  }
    
    CALLBACK_EPILOGUE

  return handled;
}

Boolean InvView_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
  FieldPtr fld;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    /* mine */
    /** place any form-initialization stuff here **/
    init_inventory_view(frm, inventory_not_messages, sotu);
    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_iv));
    update_field_scrollers2(frm, fld, repeat_iv_up, repeat_iv_down);
    if (!inventory_not_messages)
      FrmCopyTitle(frm, "Message Log");
    /* end mine */	
    FrmDrawForm(frm);
    handled = true;
    break;

  case ctlSelectEvent:
    switch(e->data.ctlSelect.controlID) {
    case btn_iv_ok:
      LeaveForm();
      handled = true;
      break;
    }
    break;

  case ctlRepeatEvent:
    /*     "Repeating controls don't repeat if handled is set true." */
    frm = FrmGetActiveForm();
    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_iv));
    switch(e->data.ctlRepeat.controlID) {
    case repeat_iv_up:
      FldScrollField(fld, 1, up);
      update_field_scrollers2(frm, fld, repeat_iv_up, repeat_iv_down);
      break;
    case repeat_iv_down:
      FldScrollField(fld, 1, down);
      update_field_scrollers2(frm, fld, repeat_iv_up, repeat_iv_down);
      break;
    }
    break;

  case keyDownEvent:
    /* hardware button -- or else graffiti. */
    frm = FrmGetActiveForm();
    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_iv));
    switch(e->data.keyDown.chr) {
    case pageUpChr:
      page_scroll_field2(frm, fld, up);
      update_field_scrollers2(frm, fld, repeat_iv_up, repeat_iv_down);
      handled = true;
      break;
    case pageDownChr:
      page_scroll_field2(frm, fld, down);
      update_field_scrollers2(frm, fld, repeat_iv_up, repeat_iv_down);
      handled = true;
      break;
    }
    break;

  default:
    break;
  }

    CALLBACK_EPILOGUE

  return handled;
}



Boolean InvSelect_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  Boolean more;
  FormPtr frm;
  ControlPtr pushbtn;
  ListPtr lst;
  Short i, selected_btn;
  Short finger;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    /* mine */
    /** place any form-initialization stuff here **/
    init_inventory_select(frm, task, sotu);
    /* end mine */	
    FrmDrawForm(frm);
    toggle_ring_buttons(frm, true, sotu); // must be _after_ form is drawn
    handled = true;
    break;

  case lstSelectEvent:
    frm = FrmGetActiveForm();
    lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, list_i));
    inventory_item = e->data.lstSelect.selection;
    if (task != INV_IDENTIFY)
      specialize_labels(frm, inventory_item, sotu);
    handled = true;
    break;
  case ctlSelectEvent:
    frm = FrmGetActiveForm();
    switch(e->data.ctlSelect.controlID) {
    case btn_i_ok:
      free_inventory_select(frm);
      LeaveForm();
      /* It is important to do 'switch' AFTER LeaveForm for msg to display */
      if (inventory_item != -1) {
	if (task == INV_IDENTIFY) {
	  /* a total hack - will only be prompted by INV_READ */
	  idntfy(inventory_item, sotu);
	  task = 0;
	  // "return;"
	} else {
	  for (i = pbtn_if_frob, selected_btn = 0 ; i <= pbtn_if_wield ; i++) {
	    pushbtn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, i));
	    if (CtlGetValue(pushbtn))
	      selected_btn = i;
	  }
	  task = do_what_i_mean(inventory_item, selected_btn, sotu);
	  switch (task) {
	  case INV_DROP:
	    if (check_drop_item())
	      drop(inventory_item); // make sure it's not cursed + there's room
	    break;
	  case INV_THROW:
	  case INV_ZAP:
	    /* leave 'inventory_item' set, go on to set 'direction'
	       and have that form "do the right thing" */
	    direction = NO_DIRECTION;
	    FrmPopupForm(DirectionForm);
	    break;
	  case INV_WIELD:
	    if (rogue->weapon && rogue->weapon->is_cursed)
	      message("already wielding a cursed weapon", sotu);
	    else
	      wield(inventory_item);
	    break;
	  case INV_EAT:
	    if (eat(inventory_item, sotu))
	      reg_move();
	    break;
	  case INV_QUAFF:
	    if (quaff(inventory_item, sotu))
	      reg_move();
	    break;
	  case INV_READ:
	    if (read_scroll(inventory_item, sotu, &more))
	      reg_move();
	    if (more) {
	      task = INV_IDENTIFY; // here we go again
	      FrmPopupForm(InvSelectForm);
	    }
	    break;
	  case INV_REMOVE:
	    remove_ring(inventory_item, sotu);  // rewritten...
	    reg_move();
	    break;
	  case INV_PUTON:
	    // this may require consulting the other set of pushbuttons
	    // I need to rewrite "put_on_ring" !!!
	    finger = get_ring_finger(frm, sotu);
	    if (finger != -1) {
	      if (put_on_ring(inventory_item, finger, sotu))
		reg_move();
	    }
	    //	    else if (sotu->r_rings > 0) {
	    //	      message("already wearing eight rings", sotu);	      
	    //	    } else {
	    //	      message("you must select a finger", sotu);
	    //	    }
	    break;
	  case INV_TAKEOFF:
	    take_off();
	    break;
	  case INV_WEAR:
	    if (rogue->armor) {
	      take_off(); // see whether this works at all!!!....
	    }
	    wear(inventory_item);
	    break;
	  default:
	    // default is to Do Nothing.
	    message("doing nothing", sotu);
	    break;
	  } // end switch(task)
	} // end else(not identify)
      } // end make-sure-there's-an-item-selected
      handled = true;
      break; // end of btn_i_ok !!!!
    case btn_i_cancel:
      free_inventory_select(frm);
      LeaveForm();
      handled = true;
      break;
    case pbtn_if_frob:
      handled = true;
      break;
    case pbtn_if_drop:
      //      task = INV_DROP;
      //      toggle_ring_buttons(frm, true, false, rogue);
      handled = true;
      break;
    case pbtn_if_throw:
      //      task = INV_THROW;
      //      toggle_ring_buttons(frm, true, false, rogue);
      handled = true;
      break;
    case pbtn_if_wield:
      //      task = INV_WIELD;
      //      toggle_ring_buttons(frm, true, false, rogue);
      handled = true;
      break;
    }
    break;

  default:
    break;
  }

    CALLBACK_EPILOGUE

  return handled;
}

// in my_object.c
void init_wiz_list2(FormPtr frm, Short some_list_id, Short object_type);
Short wiz_form_object_type = -1;
Boolean Wiz_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
  Short whatkind = -1;
  ListPtr lst;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    init_wiz_list2(frm, list_wiza, -1);
    wiz_form_object_type = -1;
    FrmDrawForm(frm);
    handled = true;
    break;

  case ctlSelectEvent:
    frm = FrmGetActiveForm();
    switch(e->data.ctlSelect.controlID) {
    case btn_wiz_ok:
      if (wiz_form_object_type != -1) {
	lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, list_wiza));
	whatkind = LstGetSelection(lst);
      }
      LeaveForm();
      /* It is important to do this after LeaveForm for msg to display */
      if (wiz_form_object_type != -1)
	new_object_for_wizard(wiz_form_object_type, whatkind, sotu);
      handled = true;
      break;
    case btn_wiz_cancel:
      LeaveForm();
      handled = true;
      break;
    }
    break;

  case lstSelectEvent:
    if (e->data.lstSelect.listID == list_wiza) {
      if (wiz_form_object_type == -1) {
	wiz_form_object_type = e->data.lstSelect.selection;
	frm = FrmGetActiveForm();
	init_wiz_list2(frm, list_wiza, wiz_form_object_type);
	FrmDrawForm(frm);
      }
    }
    handled = true;
    break;


  default:
    break;
  }

    CALLBACK_EPILOGUE

  return handled;
}

void draw_circle(Short x, Short y, Short radius, Boolean b); /* in chuvmey */
void usr_form_init2(FormPtr frm, struct state_of_the_union *sotu);
void usr_form_update2(FormPtr frm, struct state_of_the_union *sotu);
static void writeRoguePrefs();
Boolean Bullseye_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
  ListPtr lst;
  ControlPtr checkbox;
  static Short rw = 0;
  static Short ws = 0;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    rw = max(1, run_walk_border - walk_search_border);
    ws = max(1, walk_search_border);
    /* and checkboxes */
    checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_1));
    CtlSetValue(checkbox, (run_on ? 1 : 0));
    checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_2));
    CtlSetValue(checkbox, (save_on_exit ? 1 : 0));
    checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_3));
    CtlSetValue(checkbox, (dave_mode ? 1 : 0));
    checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_4));
    CtlSetValue(checkbox, (scroll_mode ? 1 : 0));
    /* set player name */
    usr_form_init2(frm, sotu);
    FrmDrawForm(frm);
    /* set initial settings for lists */
    lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, list_bul_1));
    LstSetSelection(lst, ws-1);
    lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, list_bul_2));
    LstSetSelection(lst, rw-1);
    handled = true;
    break;

  case lstSelectEvent:
    switch (e->data.lstSelect.listID) {
    case list_bul_1:
      ws = max(1, 1 + e->data.lstSelect.selection);      /* "hole" */
      break;
    case list_bul_2:
      rw = max(1, 1 + e->data.lstSelect.selection);      /* "donut" */
      break;
    }
    handled = true;
    break;
  case ctlSelectEvent:
    frm = FrmGetActiveForm();
    switch(e->data.ctlSelect.controlID) {
      /*     case btn_bul_save: */
      /*       handled = true; */
      /*       break; */
    case btn_bul_ok:
      run_walk_border = rw+ws;
      walk_search_border = ws;
      /* read run/walk checkbox... */
      checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_1));
      run_on = (CtlGetValue(checkbox) != 0);	
      /* read save/don't checkbox... */
      checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_2));
      save_on_exit = (CtlGetValue(checkbox) != 0);	
      /* read dave-mode checkbox... */
      checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_3));
      dave_mode = (CtlGetValue(checkbox) != 0);	
      /* read scroll-mode checkbox... */
      checkbox = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, check_bul_4));
      scroll_mode = (CtlGetValue(checkbox) != 0);	
      /* read player name... */
      usr_form_update2(frm, sotu);
      /* write bools and shorts to a PREFERENCES data thingy. */
      writeRoguePrefs();
      LeaveForm();
      /* It is important to do this after LeaveForm for msg to display */
      /* <anything with message> */
      handled = true;
      break;
    case btn_bul_cancel:
      LeaveForm();
      handled = true;
      break;
    case btn_bul_draw:
      draw_circle(80, 80, (rw+ws)*10, true);
      draw_circle(80, 80, ws*10, false);
      handled = true;
      break;
    case btn_bul_clear:
      draw_circle(80, 80, 160, false);
      FrmDrawForm(frm);
      handled = true;
      break;
    }
    break;

  default:
    break;
  }

    CALLBACK_EPILOGUE

  return handled;
}

Boolean About_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    FrmDrawForm(frm);
    handled = true;
    break;

  case ctlSelectEvent:
    switch(e->data.ctlSelect.controlID) {
    case btn_about_ok:
      LeaveForm();
      handled = true;
      break;
    case btn_about_more:
      FrmHelp(AboutStr);
      handled = true;
      break;
    }
    break;

  default:
    break;
  }

    CALLBACK_EPILOGUE

  return handled;
}

Boolean Map_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    /* ...init form... */
    FrmDrawForm(frm);
    /* this must be AFTER draw form */
    show_dungeon(sotu);
    handled = true;
    break;

    /*
  case ctlSelectEvent:
    switch(e->data.ctlSelect.controlID) {
    case btn_map_done:
      LeaveForm();
      handled = true;
      break;
    }
    break;
    */
  case penDownEvent:
    LeaveForm();
    handled = true;
    break;

  default:
    break;
  }

    CALLBACK_EPILOGUE

  return handled;
}

void clear_identifications2();
Boolean TopTen_Form_HandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
  FieldPtr fld;
    
    CALLBACK_PROLOGUE

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_topten));
    save_me = false;
    /* foo */
    init_topten_view(frm);
    update_field_scrollers2(frm, fld, repeat_topten_up, repeat_topten_down);
    /* end mine */
    FrmDrawForm(frm);
    handled = true;
    break;

  case ctlSelectEvent:
    frm = FrmGetActiveForm();
    switch(e->data.ctlSelect.controlID) {
    case btn_dead_ok:
      /* \begin{dangerous half-assed implementation} */
      free_old_level(sotu); /* please, no V's and M's on L1. */
      init_sotu(); /* no alloc here, now. */
      clear_identifications2();
      // that will:
      // init_id_potions to get rid of ids and 'call' titles
      // init_id_scrolls to get rid of ids and 'call' titles
      // also, might need wands and rings...
      mix_colors(); /* no alloc here */
      /* um. should there be a blue potion? i forget */
      get_wand_and_ring_materials(); /* no alloc here */
      /* (does anything else need to be cleared???) */
      /* so, er, something's not right with level_monsters
	 (and possibly level_objects, I can't tell.)
	 You go down 20 levels, get killed, hit "play again",
	 and the V's and so on are stuck where you_1 left 'em,
	 in the concrete or maybe where they can stomp you_2. */
      do_make_new_level = true;
      save_me = true;
      i_am_dead = false;
      LeaveForm();
      FrmGotoForm(MainForm); /* but all the forms will pile up... */
      check_message();
      /* \end{dangerous half-assed implementation} */
      /* Well.... it seems to work... */
      handled = true;
      break;
    }
    break;

  case ctlRepeatEvent:
    /*     "Repeating controls don't repeat if handled is set true." */
    frm = FrmGetActiveForm();
    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_topten));
    switch(e->data.ctlRepeat.controlID) {
    case repeat_topten_up:
      FldScrollField(fld, 1, up);
      update_field_scrollers2(frm, fld, repeat_topten_up, repeat_topten_down);
      break;
    case repeat_topten_down:
      FldScrollField(fld, 1, down);
      update_field_scrollers2(frm, fld, repeat_topten_up, repeat_topten_down);
      break;
    }
    break;

  case keyDownEvent:
    /* hardware button -- or else graffiti. */
    frm = FrmGetActiveForm();
    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_topten));
    switch(e->data.keyDown.chr) {
    case pageUpChr:
      page_scroll_field2(frm, fld, up);
      update_field_scrollers2(frm, fld, repeat_topten_up, repeat_topten_down);
      handled = true;
      break;
    case pageDownChr:
      page_scroll_field2(frm, fld, down);
      update_field_scrollers2(frm, fld, repeat_topten_up, repeat_topten_down);
      handled = true;
      break;
    }
    break;


  default:
    break;
  }

    CALLBACK_EPILOGUE

  return handled;
}

// usr_form_* moved to chuvmey_nolib


/*****************************************************************************
 *                                                                           *
 *   The following stufff actually belongs in this file.                     *
 *                                                                           *
 *****************************************************************************/
/*****************************************************************************
 *                                                                           *
 *                      OpenDatabase                                         *
 *                                                                           *
 *****************************************************************************/
static Boolean OpenDatabase(void)
{
  do_make_new_level = false;
  /* Try to open the Rogue database.
     If it does not exist, create one. */

  RogueDB =  DmOpenDatabaseByTypeCreator(RogueDBType,
					 RogueAppID,
					 dmModeReadWrite);
  if (!RogueDB) {
    if (DmCreateDatabase(0, // card number 0
			 RogueDBName,
			 RogueAppID,
			 RogueDBType,
			 false))
      return 1;
    RogueDB = DmOpenDatabaseByTypeCreator(RogueDBType,
					   RogueAppID,
					   dmModeReadWrite);
    /* Success has crowned my efforts!  and I may consider myself
       engaged to Phyllis!  ok, maybe not. */
    do_make_new_level = true;    
    create_top_ten_record();
    return 0;
  }

  if (DmNumRecords(RogueDB) <= 10)
    do_make_new_level = true; /* the database doesn't really have stuff */    

  /* there was a Rogue DB and we opened it */
  return 0;
}


/*****************************************************************************
 *                      ApplicationHandleEvent                               *
 *****************************************************************************/
static Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    Word    formId;
    Boolean handled = false;

    if (e->eType == frmLoadEvent) {
	formId = e->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	switch(formId) {
	case MainForm:
	    FrmSetEventHandler(frm, Main_Form_HandleEvent);
	    break;
	case DirectionForm:
	    FrmSetEventHandler(frm, Direction_Form_HandleEvent);
	    break;
	case InvViewForm:
	    FrmSetEventHandler(frm, InvView_Form_HandleEvent);
	    break;
	case InvSelectForm:
	    FrmSetEventHandler(frm, InvSelect_Form_HandleEvent);
	    break;
	case WizForm:
	    FrmSetEventHandler(frm, Wiz_Form_HandleEvent);
	    break;
	case BullseyeForm:
	    FrmSetEventHandler(frm, Bullseye_Form_HandleEvent);
	    break;
	case AboutForm:
	    FrmSetEventHandler(frm, About_Form_HandleEvent);
	    break;
	case MapForm:
	    FrmSetEventHandler(frm, Map_Form_HandleEvent);
	    break;
	case TopTenForm:
	    FrmSetEventHandler(frm, TopTen_Form_HandleEvent);
	    break;
	}
	handled = true;
    }

    return handled;
}

static void readRoguePrefs() {
  Word prefsSize;
  RoguePreferenceType prefs;
  prefsSize = sizeof(RoguePreferenceType);
  if (PrefGetAppPreferences(RogueAppID, RogueAppPrefID, 
			    &prefs, &prefsSize, true) != noPreferenceFound) {
    /* ok set some globals */
    run_on = prefs.run_on;
    save_on_exit = prefs.save_on_exit;
    dave_mode = prefs.dave_mode;
    scroll_mode = prefs.scroll_mode;
    run_walk_border = max(prefs.run_walk_border, 1);
    walk_search_border = max(prefs.walk_search_border, 1);
    /* also set default name */
    /* also set wizard? */
  }
}
static void writeRoguePrefs() {
  Word prefsSize;
  RoguePreferenceType prefs;
  prefsSize = sizeof(RoguePreferenceType);

  prefs.run_on = run_on;
  prefs.save_on_exit = save_on_exit;
  prefs.dave_mode = dave_mode;
  prefs.scroll_mode = scroll_mode;
  prefs.run_walk_border = run_walk_border;
  prefs.walk_search_border = walk_search_border;
  PrefSetAppPreferences(RogueAppID, RogueAppPrefID, RogueAppPrefVersion,
			&prefs, prefsSize, true);
}

/*****************************************************************************
 *                      StartApplication                                     *
 *****************************************************************************/
/* moved to the head of the class */
static Word StartApplication(void)
{
  Word error;
  error = OpenDatabase();
  if (error) return error;
/*   error = checkAppInfo(); */
/*   if (error) return error; */

  /* DO NOT START if there is no rogue library loaded.  so there. */
  if (! DmFindDatabase(0, RogueLibraryName))
    return 1;

  /* load game and/or do all the ONE-TIME initialization */
  init_game();

  readRoguePrefs();

  FrmGotoForm(MainForm);
  return 0;
}

/*****************************************************************************
 *                      StopApplication                                      *
 *****************************************************************************/
/* Save preferences, close forms, close app database */
extern Char * hit_message;
extern Boolean *rooms_visited;
#define Free_me(a)  h = MemPtrRecoverHandle((a)); if (h) MemHandleFree(h);
static void free_everything() {
  VoidHand h;
  Short i;

  free_old_level(sotu);
  Free_me(sotu->level_objects);
  Free_me(sotu->level_monsters);

  Free_me(hit_message);
  Free_me(rooms_visited);

  //  Free_me(sotu->fruit);
  Free_me(sotu->level_points);
  //  for (i = 0 ; i < TRAPS*2 ; i++) {
  //    Free_me(sotu->trap_strings[i]);
  //  }
  for (i = 0 ; i < DROWS ; i++) {
    Free_me(sotu->dungeon[i]);
  }
  Free_me(sotu->dungeon);
  Free_me(sotu->rooms);
  Free_me(sotu->traps);
  Free_me(sotu->id_potions);
  Free_me(sotu->id_scrolls);
  Free_me(sotu->id_weapons);
  Free_me(sotu->id_armors);
  Free_me(sotu->id_rings);
  Free_me(sotu->id_wands);
  Free_me(sotu->is_wood);

  Free_me(sotu->roguep);
  Free_me(sotu);

}
void save_character2(struct state_of_the_union *sotu); // in monster.c
static void StopApplication(void)
{
  if (save_me && save_on_exit) {
    check_message(); // force clear
    sotu->last_old_message_shown = SAVED_MSGS-1; // no "--more--"
    message("Saving...", sotu);
    save_character2(sotu); /* save if not DEAD. */
  }
  
  FrmSaveAllForms();
  FrmCloseAllForms();

  /* it might be nice to CLOSE the DATABASE.  not necessary? */
  if (RogueDB!=NULL) {
    DmCloseDatabase(RogueDB);
    RogueDB = NULL;
  }
  free_everything();
  /* it might be nice to free some crap, too */
}


/*****************************************************************************
 *                      EventLoop                                            *
 *****************************************************************************/

/* The main event loop */


static void EventLoop(void)
{
    Word err;
    EventType e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (! SysHandleEvent (&e))
	    if (! MenuHandleEvent (NULL, &e, &err))
		if (! ApplicationHandleEvent (&e))
		    FrmDispatchEvent (&e);
    } while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
/*****************************************************************************
 *                      PilotMain                                            *
 *****************************************************************************/
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    Word err;

    /*
#if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL
    Word crc = Crc16CalcBlock(NULL, 256, 0);
#endif
*/
/* #if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL */
/*     DWord startFreeDynamicMemory, endFreeDynamicMemory, junk; */
/*     MemHeapFreeBytes(0, &startFreeDynamicMemory, &junk); */
/* #endif */

    if (cmd == sysAppLaunchCmdNormalLaunch) {

      err = StartApplication();
      if (err) return err;

      EventLoop();
      StopApplication();

    } else {
      return sysErrParamErr;
    }
    /*
#if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL
    ErrNonFatalDisplayIf (crc != Crc16CalcBlock(NULL, 256, 0), 
			  "low Memory Trashed)");
#endif
*/
/* #if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL */
/*     MemHeapFreeBytes(0, &endFreeDynamicMemory, &junk); */
/*     ErrNonFatalDisplayIf (startFreeDynamicMemory != endFreeDynamicMemory, */
/* 			  "Memory Leak Occurred"); */
/* #endif */

    return 0;
}

/*******************************************************************
                    add new stuff below
*******************************************************************/

 
