/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/
/*
 * inventory.c
 *
 * This source herein may be modified and/or distributed by anybody who
 * so desires, with the following restrictions:
 *    1.)  No portion of this notice shall be removed.
 *    2.)  Credit shall not be taken for the creation of this source.
 *    3.)  This code is not to be traded, sold, or used for personal
 *         gain or profit.
 *
 */

#include "Pilot.h"
#include "callback.h"
#include "iRogueRsc.h"
#include "Globals.h"

#include "rogue.h"

Char * md_malloc2(Int n);
void get_desc2(object *obj, Char *desc, struct state_of_the_union *sotu);
void get_desc_brief2(object *obj, Char *desc, struct state_of_the_union *sotu);
Int get_rand2(Int x, Int y);

/* possibly not large enough */
#define BIG_NUMBER 2048

extern struct state_of_the_union * sotu;

//const Char *wand_materials[WAND_MATERIALS]
// added some metals and woods from urogue
const Char wand_materials[WAND_MATERIALS][12] = {
  "steel ",
  "bronze ",
  "gold ",
  "silver ",
  "copper ",
  "nickel ",
  "cobalt ",
  "tin ",
  "iron ",
  "magnesium ",
  "chrome ",
  "carbon ",
  "platinum ",
  "silicon ",
  "titanium ",
  "aluminum ",
  "bone ",
  "brass ",
  "lead ",
  "pewter ",
  "zinc ",
  "ivory ",
  "arenak ",

  "teak ",
  "oak ",
  "cherry ",
  "birch ",
  "pine ",
  "cedar ",
  "redwood ",
  "balsa ",
  "walnut ",
  "maple ",
  "mahogany ",
  "elm ",
  "palm ",
  "zebra-wood ",
  "banyan ",
  "cinnabari ",
  "dogwood ",
  "driftwood ",
  "ebony ",
  "eucalyptus ",
  "hemlock ",
  "ironwood ",
  "manzanita ",
  "rosewood "
};
// Added: alum., bone, brass, lead, pewter, zinc.
// Moved ivory to "metals".
// Added: bany, cinnabari, dogw, driftw, ebony, heml, ironw, manz, rose, eu
// Changed 'wooden' to 'zebra-wood'
// Leaving out: avocado wood, persimmon wood


// const Char *gems[GEMS]
// Added some more from urogue.
const Char gems[GEMS][16] = {
  "diamond ",
  "stibotantalite ",
  "lapis-lazuli ",
  "ruby ",
  "emerald ",
  "sapphire ",
  "amethyst ",
  "quartz ",
  "tiger-eye ",
  "opal ",
  "agate ",
  "turquoise ",
  "pearl ",
  "garnet ",

  "alexandrite ",
  "azurite ",
  "carnelian ",
  "chrysoberyl ",
  "chrysoprase ",
  "citrine ",
  "hematite ",
  "jacinth ",
  "jade ",
  "kryptonite ",
  "malachite ",
  "moonstone ",
  "obsidian ",
  "olivine ",
  "onyx ",
  "peridot ",
  "rhodochrosite ",
  "sardonyx ",
  "serpentine ",
  "spinel ",
  "topaz ",
  "tourmaline ",

  "faidon"
};
// diffs:  no "stibotantalite "; "Lapus lazuli", "Tiger eye".


// const Char *syllables[MAXSYLLABLES]
// I think I like the scrolls the way they are.
// Note urogue has a fairly different syllable set.
const Char syllables[MAXSYLLABLES][7] = {
  "blech ",
  "foo ",
  "barf ",
  "rech ",
  "bar ",
  "blech ",
  "quo ",
  "bloto ",
  "woh ",
  "caca ",
  "blorp ",
  "erp ",
  "festr ",
  "rot ",
  "slie ",
  "snorf ",
  "iky ",
  "yuky ",
  "ooze ",
  "ah ",
  "bahl ",
  "zep ",
  "druhl ",
  "flem ",
  "behil ",
  "arek ",
  "mep ",
  "zihr ",
  "grit ",
  "kona ",
  "kini ",
  "ichi ",
  "niah ",
  "ogr ",
  "ooh ",
  "ighr ",
  "coph ",
  "swerr ",
  "mihln ",
  "poxi "
};

VoidHand view_TextHandle = NULL;
//#define INV_BUF_SIZE 160
//Char inv_buf[INV_BUF_SIZE];
Char ** inventory_select_items;
Short inventory_select_numItems;
extern Short inventory_item;

//static void inventory(object *pack, UShort mask, CharPtr txtP);
//static void inventory(CharPtr txtP, struct state_of_the_union * sotu);


/**********************************************************************
                       MAKE_IT_FIT
 IN:
 buf = the string that might be too long
 avail_space = the width (in .rcp terms) of the space it goes in
 maxbuflen = the space allocated to buf
 OUT:
 nothing (may modify buf)
 PURPOSE:
 If necessary, this truncates 'buf' and adds "..." to the end.
 **********************************************************************/
void make_it_fit(CharPtr buf, Short avail_space, Short maxbuflen) {
  /* let's not overflow the space available */
  Boolean fits;
  Int length_msg, width_msg;
  length_msg = StrLen(buf);
  width_msg = avail_space;
  FntCharsInWidth(buf, &width_msg, &length_msg, &fits);
  if (!fits) {
    width_msg -= 14; /* make room for ducklings, er, "..." */
    FntCharsInWidth(buf, &width_msg, &length_msg, &fits);
    if (length_msg+5 > maxbuflen)
      length_msg -= 5; /* to be sure.  actually this will never happen? */
    buf[length_msg+1] = '.';
    buf[length_msg+2] = '.';
    buf[length_msg+3] = '.';
    buf[length_msg+4] = '\0'; /* check for FENCEPOST error */
  }
}

/**********************************************************************
                       INVENTORY
 IN:
 pack = pointer to the linked list of inventory items
 mask = in case you only want inventory items of a particular kind
 txtP = space to print each item's description in
 various globals
 OUT:
 nothing (implicitly, txtP)
 PURPOSE:
 For each inventory item corresponding to the mask, print a
 description in txtP.  (Called when the form that will display
 this text is being initialized.)
 **********************************************************************/
//static void inventory(object *pack, UShort mask, CharPtr txtP) {
// had to replace inv_buf with 'my_buf'
// so why in heck does init_inventory_select still work?  hey?
/*
static void inventory(CharPtr txtP, struct state_of_the_union * sotu) {
  object *obj;  
  Char my_buf[80];

  //  obj = pack->next_object;
  obj = sotu->roguep->pack.next_object;
  //  if (sotu->roguep->pack.next_object) return;
  //  if (pack) return;
  //  if (obj) return;
  //  if (obj->what_is & FOOD) return;
  while (obj) {
    my_buf[0] = obj->ichar;
    my_buf[1] = ((obj->what_is & ARMOR) && obj->is_protected) ? '}' : ')';
    my_buf[2] = ' ';
    get_desc2(obj, my_buf+3, sotu); // er.  will it overflow?
    if (obj->next_object)
      StrCat(my_buf, "\n");
    MemMove(txtP, my_buf, StrLen(my_buf)+1);
    txtP += StrLen(my_buf);
    obj = obj->next_object;
  }
}
*/
/**********************************************************************
                       INIT_INVENTORY_VIEW
 IN:
 frm = a form that has a text-view widget
 inv_not_msgs = I'm reusing this for inventory or messages history
 various globals
 OUT:
 nothing
 PURPOSE:
 Initializes the displayed form 'frm' with either an inventory list or
 a list of the last SAVED_MSGS messages.  (the same form is used for
 both...)  Called when frm is being initialized.
 **********************************************************************/
void init_inventory_view(FormPtr frm, Boolean inv_not_msgs,
			 struct state_of_the_union *sotu) {
  FieldPtr fld;
  CharPtr txtP;
  Short i;

  /* Get the text field 'field_iv' */
  fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, field_iv));

  /* not sure whether to try to free old handles */
  if (view_TextHandle != NULL) {
    /* MemHandleFree(view_TextHandle); // bus error.. erk.  leak? */
  }

  /* Create a mem. handle and lock it */
  view_TextHandle = MemHandleNew(BIG_NUMBER);
  txtP = MemHandleLock(view_TextHandle);

  /* Use MemMove and/or MemSet to copy text to the mem. handle */
  //  if (inv_not_msgs) {
  //    inventory(txtP, sotu);
  //  } else {
    for (i = 0 ; i < SAVED_MSGS ; i++) {
      MemMove(txtP, sotu->old_messages[i], StrLen(sotu->old_messages[i])+1);
      txtP += StrLen(sotu->old_messages[i]);
    }
    //  }

  /* Unlock the handle.  Set the field to display the handle's text. */
  MemHandleUnlock(view_TextHandle);
  FldSetTextHandle(fld, (Handle) view_TextHandle);

  /* update scrollers... oo er. */
}


// inventory() was here

/**********************************************************************
                       INIT_INVENTORY_SELECT
 IN:
 frm = the form to initialize part of
 task = either it's INV_IDENTIFY or it's not (general use)
 various globals
 OUT:
 nothing
 PURPOSE:
 This initializes part of the form that's used to list inventory items
 (and possible actions on items, if the task is not to 'identify')
 for selection by the user.
 I sense that it could stand some cleaning up.
 Do not use the inventory form if the pack is empty.
 **********************************************************************/
void init_inventory_select(FormPtr frm, Short task,
			   struct state_of_the_union *sotu) {
  ListPtr lst;
  ControlPtr pushbtn;
  Short numItems, i, k;
  object *obj;
  Char my_buf[80];
  Char cur_ichar = 'a';

  /* one little special-case */
  if (task == INV_IDENTIFY) {
    FrmCopyTitle(frm, "...identify what?");
    for (i = pbtn_if_frob ; i <= pbtn_if_wield ; i++) {
      /* drop,eat,quaff,read,wield,puton,remove,wear,takeoff,zap,throw. */
      /* the L/R is already hidden by default */
      pushbtn = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, i));
      CtlHideControl(pushbtn);
    }
  }
  // (hey what if the inventory scroll is the only item you have?)

  /* set the stuff in the list */
  numItems = 0;
  for (obj = sotu->roguep->pack.next_object ;
       obj != NULL ;
       obj = obj->next_object) {
    numItems++;
  }

  if (numItems == 0)
    ; /* XXXXXX */
  obj = sotu->roguep->pack.next_object;
  inventory_select_items = (Char **) md_malloc2(sizeof(Char *) * numItems);

  // the pack is empty!  the caller is supposed to be checking for that.
  ErrNonFatalDisplayIf ( (inventory_select_items == NULL),
			 "XXXXXX SERIOUS EVIL");

  for (i = 0 ; i < numItems && obj ; i++, obj = obj->next_object ) 
    {
      /* if (obj->what_is & mask) */
      obj->ichar = cur_ichar++; /* re-order the inventory letters */
      if (cur_ichar > 'z')
	cur_ichar = 'A';
      my_buf[0] = obj->ichar; 
      my_buf[1] = ((obj->what_is & ARMOR) && obj->is_protected) ? ']' : ')';
      my_buf[2] = ' ';
      k = 3;
      if (obj->in_use_flags & BEING_USED) {
	if ( (obj->in_use_flags & BEING_WIELDED)
	     || (obj->in_use_flags & BEING_WORN) ) {
	  my_buf[2] = my_buf[1];
	  my_buf[3] = ' ';
	  k=4;
	} else if (obj->in_use_flags & ON_LEFT_HAND) {
	  my_buf[1] = 171;
	} else if (obj->in_use_flags & ON_RIGHT_HAND) {
	  my_buf[1] = 187;
	}
      }
      get_desc_brief2(obj, my_buf + k, sotu); /* er.  will it overflow? */


/*       StrCat(my_buf, "\n"); */
      make_it_fit(my_buf, 122, 80); // 80 is size of my_buf
      /* ok done truncating */
      inventory_select_items[i] = md_malloc2(sizeof(Char)*(StrLen(my_buf)+1));

      // (malloc failed.)
      ErrNonFatalDisplayIf ( (inventory_select_items[i] == NULL),
			     "XXXXXX SERIOUS EVIL");

      StrCopy(inventory_select_items[i], my_buf);
    } /* end for */
  if (i < numItems) {
    numItems = i; /* Just In Case! */
  }
  lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, list_i));
  LstSetListChoices(lst, inventory_select_items, numItems);
  LstSetSelection(lst, -1);
  inventory_select_numItems = numItems;
  /* And just who is planning to free this memory I'd like to know? */
}

/**********************************************************************
                       FREE_INVENTORY_SELECT
 IN:
 frm = the inventory select form, again
 various globals
 OUT:
 nothing
 PURPOSE:
 This is called when the form is exited, I guess,
 and tries to free up some memory allocated by its initializer.
 **********************************************************************/
void free_inventory_select(FormPtr frm) {
  ListPtr lst;
  Short i;
  VoidHand h;
  lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, list_i));
  LstSetListChoices(lst, NULL, 0);
  for (i = 0 ; i < inventory_select_numItems ; i++) {
    h = MemPtrRecoverHandle(inventory_select_items[i]);
    if (h) 
      MemHandleFree(h);
  }
  h = MemPtrRecoverHandle(inventory_select_items);
  if (h) 
    MemHandleFree(h);
}



/* inventory(pack, mask) changed to a popup dialog */


/**********************************************************************
                       MIX_COLORS
 IN:
 various globals
 OUT:
 nothing
 PURPOSE:
 Called whenever a new rogue is started; this mixes up the colors
 assigned to the various kinds of potions.
 **********************************************************************/
/* 32 * 3 StrCopy's.... ugh */
void mix_colors() {
  Short i, j, k;
  Char t[32];

  for (i = 0; i <= 32; i++) {
    j = get_rand2(0, (POTIONS - 1));
    k = get_rand2(0, (POTIONS - 1));
    if (j != k) {
      StrCopy(t,                         sotu->id_potions[j].title);
      StrCopy(sotu->id_potions[j].title, sotu->id_potions[k].title);
      StrCopy(sotu->id_potions[k].title, t);
    }
    // "it was like that when I got here" (i.e. commented out)
    //		swap_string(id_potions[j].title, id_potions[k].title);
  }
}


/**********************************************************************
                       MAKE_SCROLL_TITLES
 IN:
 various globals
 OUT:
 nothing
 PURPOSE:
 Called whenever a new rogue is started; this makes up the titles
 assigned to the various kinds of scrolls.
 **********************************************************************/
/* Also string-intensive but I think it's ok. */
void make_scroll_titles() {
  Short i, j, n;
  Short sylls, s;

  for (i = 0; i < SCROLLS; i++) {
    sylls = get_rand2(2, 5);
    /*		id_scrolls[i].title=(char *)malloc(128);
		if(id_scrolls[i].title==(char *)0)
		clean_up("Panic: no memory.",0);*/
    StrCopy(id_scrolls[i].title, "'");

    for (j = 0; j < sylls; j++) {
      s = get_rand2(1, (MAXSYLLABLES-1));
      StrCat(id_scrolls[i].title, syllables[s]);
    }
    n = StrLen(id_scrolls[i].title);
    StrCopy(id_scrolls[i].title+(n-1), "' ");
  }
}

/************************************************
  get_desc MOVED to LibRogue/lib_OBJECT.c
************************************************/

/**********************************************************************
                       GET_WAND_AND_RING_MATERIALS
 IN:
 various globals
 OUT:
 nothing
 PURPOSE:
 Called whenever a new rogue is started (?); this mixes up the
 materials assigned to the different wand types and ring types.
 **********************************************************************/
void get_wand_and_ring_materials() {
  Short i, j;
  //  Boolean used[max(WAND_MATERIALS, GEMS)];
  Boolean used[WAND_MATERIALS];

  for (i = 0; i < WAND_MATERIALS; i++) {
    used[i] = 0;
  }
  for (i = 0; i < WANDS; i++) {
    do {
      j = get_rand2(0, WAND_MATERIALS-1);
    } while (used[j]);
    used[j] = 1;
    /*		id_wands[i].title=(char *)malloc(128);*/
    StrCopy(id_wands[i].title, wand_materials[j]);
    id_wands[i].id_status = UNIDENTIFIED;
    sotu->is_wood[i] = (j > MAX_METAL);
  }
  for (i = 0; i < GEMS; i++) {
    used[i] = 0;
  }
  for (i = 0; i < RINGS; i++) {
    do {
      j = get_rand2(0, GEMS-1);
    } while (used[j]);
    used[j] = 1;
    /*		id_rings[i].title=(char *)malloc(128);*/
    StrCopy(id_rings[i].title, gems[j]);
    id_rings[i].id_status = UNIDENTIFIED;
  }
}

/* single_inv
   perhaps this could also be a Popup  - get rid of pack_letter !    */

/******************************************************
  get_id_table MOVED to LibRogue, believe it or don't
******************************************************/

/* inv_armor_weapon - gone
   inv_armor - gone
   inv_weapon - gone   */


