/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/
/*
 * main.c
 *
 * This source herein may be modified and/or distributed by anybody who
 * so desires, with the following restrictions:
 *    1.)  No portion of this notice shall be removed.
 *    2.)  Credit shall not be taken for the creation of this source.
 *    3.)  This code is not to be traded, sold, or used for personal
 *         gain or profit.
 *
 */

#include "Pilot.h"
#include "callback.h"
#include "iRogueRsc.h"
#include "Globals.h"

#include "rogue.h"

extern struct state_of_the_union * sotu;
extern Boolean do_make_new_level;

/**********************************************************************
                       INIT_GAME
 IN:
 various globals
 OUT:
 nothing
 PURPOSE:
 Do all the allocation and initialization needed to begin a session.
 (Later, we will check for a save file, and load the character
 information from it into the allocated and initialized space.)
 **********************************************************************/
void alloc_id_arrays2();
void init_game() {
  seed_random();
  alloc_and_init_sotu();
  alloc_id_arrays2();
  mix_colors();
  get_wand_and_ring_materials();
}

/**********************************************************************
                       MAKE_NEW_LEVEL
 IN:
 various globals
 OUT:
 nothing
 PURPOSE:
 Make a new dungeon map; place objects, monsters, etc., in it.
 It is called whenever the rogue goes up/down a level (via stairs or
 trapdoor), and at the start of a session IF no save file is found
 (whatever sort of space is allocated here had better also be
 allocated by the reload function or someone will be unhappy later...)
 NOTE: This must be called AFTER the MainForm is drawn - this is
 why there is that global variable do_make_new_level kicking around.
 **********************************************************************/
/* I hope none of these allocate space that chuvmey doesn't. */
void make_new_level() {
  clear_level(sotu);
  make_level(); /* Illegal instruction: 100a */
  put_objects(); 
  put_stairs(sotu); 
  add_traps(sotu); 
  put_mons(); 
  put_player(sotu->party_room, sotu); 
  print_stats(STAT_ALL, sotu); 

  // aggravate!
  if (sotu->ring_flags & RING_AGGRAVATE) {
    if (sotu->ring_flags & RING_ESP)
      message("you sense hostility", sotu);
    // I should also do this when you put the ring on,  But I'm not.
    aggravate(sotu);
  }

}


