/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/
/*
 * pack.c
 *
 * This source herein may be modified and/or distributed by anybody who
 * so desires, with the following restrictions:
 *    1.)  No portion of this notice shall be removed.
 *    2.)  Credit shall not be taken for the creation of this source.
 *    3.)  This code is not to be traded, sold, or used for personal
 *         gain or profit.
 *
 */

/* #ifndef CURSES */
/* #include <curses.h> */
/* #endif CURSES */
#include "Pilot.h"
#include "callback.h"
#include "iRogueRsc.h"
#include "Globals.h"

#include "rogue.h"

extern struct state_of_the_union * sotu;

Char curse_message[] = "you can't, it appears to be cursed";


/************************************************
  add_to_pack MOVED to LibRogue
************************************************/

/************************************************
  take_from_pack MOVED to LibRogue
************************************************/

/**********************************************************************
                       PICK_UP
 IN:
 row,col = location of item to pick up
 status = another return value: whether object was destroyed (special case.)
 various globals
 OUT:
 the object that was picked up, for use by caller...
 PURPOSE:
 If possible, pick up an object...
 **********************************************************************/
object * pick_up(Short row, Short col, Short *status) {
  object *obj;
  Short new_weight;

  obj = object_at(level_objects, row, col);
  *status = 1;
  
  /* Special case: Scare-Monster scroll can't be picked up. */
  if ((obj->what_is == SCROLL) && (obj->which_kind == SCARE_MONSTER) &&
      obj->picked_up) {
    message("the scroll turns to dust as you pick it up", sotu);
    dungeon[row][col] &= (~OBJECT);
    vanish(obj, level_objects, sotu);
    //    reg_move(); // NO. BAD. WRONG.
    *status = 0;
    if (id_scrolls[SCARE_MONSTER].id_status == UNIDENTIFIED) {
      id_scrolls[SCARE_MONSTER].id_status = IDENTIFIED;
    }
    return(0);
  }

  /* Special case: Gold is not a separate object per se. */
  if (obj->what_is == GOLD) {
    rogue->gold += obj->quantity;
    dungeon[row][col] &= ~(OBJECT);
    take_from_pack(obj, level_objects);
    print_stats(STAT_GOLD, sotu);
    return(obj);	/* obj will be free_object()ed in one_move_rogue() */
  }

  /* Another exceptional case: pack won't hold any more objects */
  if (pack_count(obj, rogue, &new_weight) >= MAX_PACK_COUNT) {
    message("pack too full", sotu);
    return(0);
  }
  if (new_weight > max_weight(sotu)) {
    message("pack too heavy", sotu);
    return(0);
  }

  /* Remove the object from the list of stuff-sitting-around and
     add it to the list of stuff-in-pack */
  dungeon[row][col] &= ~(OBJECT);
  take_from_pack(obj, level_objects);
  obj = add_to_pack(obj, &rogue->pack, 1, sotu);
  obj->picked_up = 1;
  return(obj);
}

/**********************************************************************
                       CHECK_DROP_ITEM
 IN:
 various globals
 OUT:
 Whether the rogue can currently drop an object.
 PURPOSE:
 (duh.)
 **********************************************************************/
Boolean check_drop_item() {
  if (dungeon[rogue->row][rogue->col] & (OBJECT | STAIRS | TRAP)) {
    message("there's already something there", sotu);
    return false;
  }
  if (!rogue->pack.next_object) {
    message("you have nothing to drop", sotu);
    return false;
  }
  return true;
}
/**********************************************************************
                       DROP
 IN:
 index = of the item in the pack-list, to be dropped.
 various globals
 OUT:
 nothing
 PURPOSE:
 If possible, drop the item.  Display appropriate message.
 **********************************************************************/
void drop(Short index) {
  /* 'index' is zero-based and is the numerical index in the pack list. */
  object *obj, *new;
  Char desc[DCOLS];

  if (index < 0)
    return;

  /* check_drop should display message for these two */
  if (dungeon[rogue->row][rogue->col] & (OBJECT | STAIRS | TRAP))
    return;
  if (!rogue->pack.next_object)
    return;

  /* the inventory select form should make this impossible */
  if (!(obj = get_nth_object(index, &rogue->pack))) {
    message("no such item.", sotu);
    return;
  }

  /* now see if it's something wielded/worn/on */
  if (obj->in_use_flags & BEING_WIELDED) {
    if (obj->is_cursed) {
      message(curse_message, sotu);
      return;
    }
    unwield(rogue->weapon, rogue);
  } else if (obj->in_use_flags & BEING_WORN) {
    if (obj->is_cursed) {
      message(curse_message, sotu);
      return;
    }
    mv_aquatars();
    unwear(rogue->armor, rogue);
    print_stats(STAT_ARMOR, sotu);
  } else if (obj->in_use_flags & ON_EITHER_HAND) {
    if (obj->is_cursed) {
      message(curse_message, sotu);
      return;
    }
    un_put_on(obj, sotu); // lib_ring takes care of printing stats
  }

  /* drop that puppy */
  obj->row = rogue->row;
  obj->col = rogue->col;

  /* if it's multiple, and not a weapon, drop "one" of it */
  if ((obj->quantity > 1) && (obj->what_is != WEAPON)) {
    obj->quantity--;
    new = alloc_object(sotu);
    *new = *obj;
    new->quantity = 1;
    obj = new;
  } else {
    obj->ichar = 'Z';  /* eh? */
    take_from_pack(obj, &rogue->pack);
  }
  place_at(obj, rogue->row, rogue->col, sotu);
  StrCopy(desc, "dropped ");
  get_desc(obj, desc+8, sotu);
  message(desc, sotu);
  reg_move();
}

/************************************************
  check_duplicate MOVED to LibRogue
  next_avail_ichar MOVED to LibRogue
  ************************************************/

/**********************************************************************
                       TAKE_OFF
 IN:
 various globals
 OUT:
 nothing
 PURPOSE:
 If it's possible, take off armor.
 **********************************************************************/
void take_off() {
  Char desc[DCOLS];
  object *obj;

  if (rogue->armor) {
    if (rogue->armor->is_cursed) {
      message(curse_message, sotu);
    } else {
      mv_aquatars(); /* interesting */
      obj = rogue->armor;
      unwear(rogue->armor, rogue);
      StrCopy(desc, "was wearing ");
      get_desc(obj, desc+12, sotu);
      message(desc, sotu);
      print_stats(STAT_ARMOR, sotu);
      reg_move();
    }
  } else {
    message("not wearing any", sotu);
  }
}

/**********************************************************************
                       WEAR
 IN:
 index = of the item to be worn, in the pack-list
 various globals
 OUT:
 nothing
 PURPOSE:
 If it's possible, wear the (armor) item.  (Not possible if you're
 already wearing some - you have to take it off first.)
 **********************************************************************/
void wear(Short index) {
  object *obj;
  Char desc[DCOLS];

  /* Rogue.c should check first whether you're already wearing some */
  if (rogue->armor)
    return;

  /* the inventory select form should make this impossible */
  if (!(obj = get_nth_object(index, &rogue->pack))) {
    message("no such item.", sotu);
    return;
  }

  if (obj->what_is != ARMOR) {
    message("you can't wear that", sotu);
    return;
  }

  obj->identified = 1;
  StrCopy(desc, "wearing ");
  get_desc(obj, desc + 8, sotu);
  message(desc, sotu);
  do_wear(obj, rogue);
  print_stats(STAT_ARMOR, sotu);
  reg_move();
}

/************************************************
  unwear MOVED to LibRogue
  do_wear ditto
************************************************/


/**********************************************************************
                       WIELD
 IN:
 index = of the item to be wielded, in the pack-list
 various globals
 OUT:
 nothing
 PURPOSE:
 If it's possible, wield the item.  (Possible if you're already wielding
 something else - it will be un-wielded.
 **********************************************************************/
void wield(Short index) {
  object *obj;
  Char desc[DCOLS];

  /* ? */
  if (rogue->weapon && rogue->weapon->is_cursed) {
/*     message(curse_message, sotu); */
    return;
  }

  if (!(obj = get_nth_object(index, &rogue->pack))) {
    /* should be impossible */
    message("no such item.", sotu);
    return;
  }

  /* You can wield most things; even, say, slime-molds */
  if (obj->what_is & (ARMOR | RING)) {
    StrPrintF(desc, "you can't wield %s",
	      ((obj->what_is == ARMOR) ? "armor" : "rings"));
    message(desc, sotu);
    return;
  }

  if (obj->in_use_flags & BEING_WIELDED) {
    message("in use", sotu);
  } else {
    unwield(rogue->weapon, rogue);
    StrCopy(desc, "wielding ");
    get_desc(obj, desc + 9, sotu);
    message(desc, sotu);
    do_wield(obj, rogue);
    reg_move();
  }
}

/************************************************
  do_wield ditto
  unwield ditto
************************************************/










/* XXX I am going to DISABLE this for now
   so I don't have to revise get_input_line yet. */
/*
call_it()
{
	short ch;
	register object *obj;
	struct id *id_table;
	char buf[MAX_TITLE_LENGTH+2];

	ch = pack_letter("call what?", (SCROLL | POTION | WAND | RING));

	if (ch == CANCEL) {
		return;
	}
	if (!(obj = get_letter_object(ch))) {
		message("no such item.", 0);
		return;
	}
	if (!(obj->what_is & (SCROLL | POTION | WAND | RING))) {
		message("surely you already know what that's called", 0);
		return;
	}
	id_table = get_id_table(obj, sotu);

	if (get_input_line("call it:","",buf,id_table[obj->which_kind].title,1,1)) {
		id_table[obj->which_kind].id_status = CALLED;
		(void) strcpy(id_table[obj->which_kind].title, buf);
	}
}
*/
/*
void call_it() {
  Short ch;
  object *obj;
  struct id *id_table;
  Char buf[MAX_TITLE_LENGTH+2];

//       Make a dialog with:
//       1. scrollable select
//       2. input field.
//       3. OK/cancel.
//       maybe I can rework the inventory-frobbing dialog Yet Again.
//       on OK, see if something is selected.
//       if it's not {scrolls, potions, wands, rings}, get sassy.
//       otherwise copy into id_table...... hm.......

  ch = pack_letter("call what?", (SCROLL | POTION | WAND | RING));

  if (ch == CANCEL) {
    return;
  }
  if (!(obj = get_letter_object(ch))) {
    message("no such item.", 0);
    return;
  }
  if (!(obj->what_is & (SCROLL | POTION | WAND | RING))) {
    message("surely you already know what that's called", 0);
    return;
  }
  id_table = get_id_table(obj, sotu); // sotu->id_<foo>

  if (get_input_line("call it:","",buf,id_table[obj->which_kind].title,1,1)) {
    id_table[obj->which_kind].id_status = CALLED;
    (void) strcpy(id_table[obj->which_kind].title, buf);
  }

  // so when is title (re-)set?
  //   init_id_potions, init_id_scrolls (make_scroll_titles) need to be done.
  //   get_wand_and_ring_materials is done - ok.

}
*/

/************************************************
  pack_count MOVED to LibRogue
************************************************/

/************************************************
  mask_pack MOVED to LibRogue
************************************************/

/* is_pack_letter */
/* No longer used - since I'm getting rid of pack_letter */

/**************************************************************************
  has_amulet MOVED to LibRogue.... on second thought, no.
*************************************************************************/
// used only in level.c... made it into a macro there.
/*
Boolean has_amulet() {
  return(mask_pack(&rogue->pack, AMULET));
}
*/

void kick_into_pack() {
  object *obj;
  Char desc[DCOLS];
  Short n, status;

  if (sotu->levitate || (sotu->ring_flags & RING_LEVITATE)) {
    message("you're floating in the air!", sotu);
    return;
  }
  if (!(dungeon[rogue->row][rogue->col] & OBJECT)) {
    message("nothing here", sotu);
    return;
  }
  if ((obj = pick_up(rogue->row, rogue->col, &status))) {
    get_desc(obj, desc, sotu);
    if (obj->what_is == GOLD) {
      free_object(obj, sotu);
    } else {
      n = StrLen(desc);
      desc[n] = '(';
      desc[n+1] = obj->ichar;
      desc[n+2] = ')';
      desc[n+3] = 0;
    }
    message(desc, sotu);
  }
  if (obj || (!status))
    reg_move();
}
/*
void kick_into_pack() {
  object *obj;
  Char desc[DCOLS];
  Short n, stat;

  if (!(dungeon[rogue->row][rogue->col] & OBJECT)) {
    message("nothing here", 0);
  } else {
    if (obj = pick_up(rogue->row, rogue->col, &stat)) {
      get_desc(obj, desc);
      if (obj->what_is == GOLD) {
	message(desc, 0);
	free_object(obj, sotu);
      } else {
	n = strlen(desc);
	desc[n] = '(';
	desc[n+1] = obj->ichar;
	desc[n+2] = ')';
	desc[n+3] = 0;
	message(desc, 0);
      }
    }
    if (obj || (!stat)) {
      reg_move();
    }
  }
}
*/
