/*********************************************************************
 * iRogue - Rogue adapted for the PalmPilot.                         *
 * Copyright (C) 1999 Bridget Spitznagel                             *
 * Note: This program is derived from rogue 5.3-clone for Linux.     *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License       *
 * as published by the Free Software Foundation; either version 2    *
 * of the License, or (at your option) any later version.            *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along with this program; if not, write to                         *
 * The Free Software Foundation, Inc.,                               *
 * 59 Temple Place - Suite 330,                                      *
 * Boston, MA  02111-1307, USA.                                      *
 *********************************************************************/
/*
 * use.c
 *
 * This source herein may be modified and/or distributed by anybody who
 * so desires, with the following restrictions:
 *    1.)  No portion of this notice shall be removed.
 *    2.)  Credit shall not be taken for the creation of this source.
 *    3.)  This code is not to be traded, sold, or used for personal
 *         gain or profit.
 *
 */

#include "Pilot.h"
#include "callback.h"
#include "iRogueRsc.h"
#include "Globals.h"

#include "rogue.h"

extern struct state_of_the_union * sotu;
extern Boolean i_am_dead;
//extern Char *you_can_move_again;

/************************************************
  quaff MOVED to LibRogue
  also Char* strange_feeling.
************************************************/

/**********************************************************************
                       READ_SCROLL
 IN:
 index = of object in rogue's pack (considered as a list)
 sotu = various globals
 identfy = another return value: is it an "identify" scroll.
 OUT:
 Was the action legal (should a clock tick be performed)
 (Also, was the scroll an "identify" scroll.)
 PURPOSE:
 Various interesting effects from reading a scroll.
 **********************************************************************/
Boolean read_scroll(Short index, struct state_of_the_union *sotu,
		    Boolean * identfy) 
{
  object *obj;
  Char msg[DCOLS];
  fighter *rogue = sotu->roguep;
  Char buf[20];

  *identfy = false;
  if (sotu->blind) {
    message("You can't see to read the scroll.", sotu);
    return false;
  }
  if (!(obj = get_nth_object(index, &rogue->pack))) {
    message("no such item.", sotu);
    return false;
  }
  if (obj->what_is != SCROLL) {
    /* I could prevent this by pruning the presented inventory */
    message("you can't read that", sotu);
    return false;
  }
  switch(obj->which_kind) {
  case SCARE_MONSTER:
    /* purpose of this scroll is, monsters can't move onto it. */
    message("you hear a maniacal laughter in the distance", sotu);
    break;
  case HOLD_MONSTER:
    hold_monster(sotu);
    break;
  case ENCH_WEAPON:
    if (rogue->weapon) {
      if (rogue->weapon->what_is == WEAPON) {
	copy_name_of(rogue->weapon, buf, sotu);
	StrPrintF(msg, "your %sglow%s %sfor a moment",
		  buf,
		  ((rogue->weapon->quantity <= 1) ? "s" : ""),
		  get_ench_color(sotu));
	message(msg, sotu);
	if (coin_toss()) {
	  rogue->weapon->hit_enchant++;
	} else {
	  rogue->weapon->d_enchant++;
	}
      }
      rogue->weapon->is_cursed = 0;
    } else {
      message("your hands tingle", sotu);
    }
    break;
  case ENCH_ARMOR:
    if (rogue->armor) {
      StrPrintF(msg, "your armor glows %sfor a moment",
		get_ench_color(sotu));
      message(msg, sotu);
      rogue->armor->d_enchant++;
      rogue->armor->is_cursed = 0;
      print_stats(STAT_ARMOR, sotu);
    } else {
      message("your skin crawls", sotu);
    }
    break;
  case IDENTIFY:
    message("this is a scroll of identify", sotu);
    obj->identified = 1;
    sotu->id_scrolls[obj->which_kind].id_status = IDENTIFIED;
    *identfy = true; /* through a major hack, we revisit inventory-select */
    break;
  case TELEPORT:
    tele(sotu);
    break;
  case SLEEP:
    if (!(sotu->ring_flags & RING_ALERTNESS)) {
      message("you fall asleep", sotu);
      take_a_nap();
    }
    break;
  case PROTECT_ARMOR:
    if (rogue->armor) {
      message( "your armor is covered by a shimmering gold shield", sotu);
      rogue->armor->is_protected = 1;
      rogue->armor->is_cursed = 0;
    } else {
      message("your acne seems to have disappeared", sotu);
    }
    break;
  case REMOVE_CURSE:
    message((!sotu->halluc) ?
	    "you feel as though someone is watching over you" :
	    "you feel in touch with the universal oneness", sotu);
    uncurse_all(sotu);
    break;
  case CREATE_MONSTER:
    create_monster();
    break;
  case AGGRAVATE_MONSTER:
    message("you hear a high pitched humming noise", sotu);
    aggravate(sotu);
    break;
  case MAGIC_MAPPING:
    message("this scroll seems to have a map on it", sotu);
    draw_magic_map(sotu); /* XXX maybe make this pop up the overview map */
    break;
  case CREATE_OBJ:
    message("you have the power to create:", sotu);
    FrmPopupForm(WizForm); // TESTING
  }
  if (sotu->id_scrolls[obj->which_kind].id_status != CALLED) {
    sotu->id_scrolls[obj->which_kind].id_status = IDENTIFIED;
  }
  vanish(obj, &rogue->pack, sotu);
  /* This determines whether the caller should call reg_move...
     basically, the caller should except when it was a SLEEP scroll */
  return (obj->which_kind != SLEEP);
}

/*************************************************
  vanish MOVED to LibRogue 
  potion_heal MOVED to LibRogue
************************************************/

/**********************************************************************
                       IDNTFY
 IN:
 index = of object in pack (consider as list)
 sotu = various globals
 OUT:
 nothing
 PURPOSE:
 Identify an item...
 **********************************************************************/
void idntfy(Short index, struct state_of_the_union *sotu) {
  object *obj;
  struct id *id_table;
  Char desc[DCOLS];
  fighter *rogue = sotu->roguep;

  if (!(obj = get_nth_object(index, &rogue->pack))) {
    message("no such item.", sotu);
    return;
  }
  obj->identified = 1;
  if (obj->what_is & (SCROLL | POTION | WEAPON | ARMOR | WAND | RING)) {
    id_table = get_id_table(obj, sotu);
    id_table[obj->which_kind].id_status = IDENTIFIED;
  }
  get_desc(obj, desc, sotu);
  message(desc, sotu);
}


/************************************************
  eat MOVED to LibRogue
************************************************/


/************************************************* 
   MOVED to LibRogue:
   hold_monster()
   tele()
   hallucinate()
   unhallucinate()
   unblind()
   relight()
*************************************************/

/**********************************************************************
                       TAKE_A_NAP
 IN:
 various globals
 OUT:
 nothing
 PURPOSE:
 Move monsters (but, like sleeping beauty, DON'T tick the rogue's clock)
 until rogue wakes.  This may be called by monsters with paralyze
 or sleep gas attacks; I've fixed it so that they aren't cumulative,
 cause that would most likely be the Kiss of Death.
 **********************************************************************/
void take_a_nap() {
  static Boolean napping = false;
  Short i;
  Long delay = SysTicksPerSecond();

  if (napping) return; // (to prevent needless cruelty)
  napping = true;
  i = get_rand(2, 5);
  SysTaskDelay(delay); // there should be some "visible delay"

  while (i--) {
    mv_mons();
    if (i_am_dead)
      return; // stop beating on a dead rogue!
  }
  SysTaskDelay(delay); // there should be some "visible delay", again
  message("you can move again", sotu);
  napping = false;
}

/************************************************* 
   MOVED to LibRogue:
   go_blind() 
   get_ench_color() 
   confuse()
   unconfuse()
   uncurse_all() 
*************************************************/
